/*
 * debug.c
 *
 * Created 7/20/94 by Chuck McManis
 * 
 * This simple module is used to do really basic debugging on the
 * miniboard. It consists of a single function, flash, that will
 * flash an LED attached to one of the port C pins. I typically
 * use a low power (10mA) set of 4 red LEDs which are connected
 * with in-line 1K resistors between the 5 volt bus and the connector.
 * sort of like :   1K          LED  
 *	5v -------\/\/\/\-------|>|----- PORTC Pin 0 - 3.
 * When connected you can add the statement "flash(3,10)" for example
 * and it will briefly illuminate the LED attached to input pin 3.
 * Using a few of these allows you to have a small diagnostic console
 * that requires practically nothing else be working right. ;-)
 * 
 * To use it in your program turn all of the LEDs off with 
 *	for (i=0;i<8;i++) flash(i,0);
 */
#include <hc11.h>

void
flash(pin, dur)
	int	pin;	/* Pin on Port C to flash */
	int	dur;	/* duration of the flash */
{
	int	i, j;

	DDRC = 0xff;			/* make portC an output port */
	pin = 1 << pin;			/* Create bit mask */
	PORTC &= ~(1 << pin);		/* Turn "on" the LED */
	for (i = 0; i < dur; i++)
		for (j = 0; j < 1000; j++)
			;	/* waste some time */

	PORTC |= (1 << pin);	/* Turn "off" the LED */
	for (i = 0; i < dur; i++)
		for (j = 0; j < 1000; j++)
			;	/* waste some time */
	return;
}


