/*
 * mbmain.c
 *
 * Created 7/21/94 by Chuck McManis
 * 
 * Revision history:
 *	7/23/94 - Took out interrupt service routines and put them in
 *		mboard.h
 *	8/04/94 - Added calls to ic2_svc and ic1_svc to support encoders
 *		which are documented in the ..\pid directory.
 * Base module for the minilib library this file contains the system clock
 * interrupt service routine.
 */

#include <hc11.h>	/* register definitions */
#include <int32.h>

/*
 * Variables - "standard" miniboard variables.
 */
INT32	time;		/* a time value */

/*
 * Standard interrupt handlers:
 *	beep_svc : deal with 'tones' on the beeper port.
 *	tick_svc : system clock tick (every millesecond)
 *	irqi_svc : IRQ Button service
 */
void	tick_svc();
extern void beep_svc(), do_motors(), do_servos(), opt_init(), irqi_svc();
extern void ic1_svc(), ic2_svc(), do_userint();

/*
 * mb_init()
 * 
 * This function is called from the startup (crt11.s) before interrupts have
 * been turned on but after the stack has been set up.
 */
void
mb_init()
{

	/* Initialize system variables */
	HI_WORD(time) = LO_WORD(time) = 0;
	/* Initialize options selected */
	opt_init();

	/* System timer flag, enable interrupt from OC4 (timer) */
	TFLG1 = TMSK1 = 0x10;

	return;
}

#pragma interrupt_handler tick_svc

void
tick_svc()	/* System time interrupt routine */
{
	unsigned int	i;

	asm("	sei");
	i = TOC4 + 2000;
	TOC4 = i;
	if (++LO_WORD(time) == 0)
		++HI_WORD(time);
	asm(" ldx #0x1000");		/* Point X at the I/O page */
	asm(" bclr  0x23,x,#0xef");	/* Re-enable this interrupt */

	/* every 16 mSec check to see if we should position any servos */
	if ((LO_WORD(time) & 0xf) == 0)
		do_servos();

	do_userint();
	/* Now do the motors */
	do_motors();
	asm(" cli");	/* re-enable interrupts */
}

#pragma abs_address:0xffd6
extern void _start();	/* entry point in crt11.s */
void (*interrupt_vectors[])() = 
	{
	(void (*)())0xffff,	/* SCI */
	(void (*)())0xffff,	/* SPI */
	(void (*)())0xffff,	/* PAIE */
	(void (*)())0xffff,	/* PAO */
	(void (*)())0xffff,	/* TOF */
	beep_svc,	/* TOC5 */
	tick_svc,	/* TOC4 */
	(void (*)())0xffff,	/* TOC3 */
	(void (*)())0xffff,	/* TOC2 */
	(void (*)())0xffff,	/* TOC1 */
	(void (*)())0xffff,	/* TIC3 */
	ic2_svc,	/* TIC2 */
	ic1_svc,	/* TIC1 */
	(void (*)())0xffff,	/* RTI */
	irqi_svc,	/* IRQ */
	(void (*)())0xffff,	/* XIRQ */
	(void (*)())0xffff,	/* SWI */
	(void (*)())0xffff,	/* ILLOP */
	(void (*)())0xffff,	/* COP */
	(void (*)())0xffff,	/* CLM */
	_start	/* RESET */
	};
#pragma end_abs_address
