/*
 *	motor.c
 *	
 * Created 7/20/94 by Chuck McManis
 *
 * This module provides the functions that determine motor speed. They are
 * used to set the motor speed to some value between 0 (off) and 16 (full
 * on.) 
 */

extern unsigned int speed[];
extern unsigned char motor_ctl;

/* Table of motor speeds */
static const unsigned int speed_table[17] = {
	0x0000,		/* %0000000000000000	* 0/16 */
	0x0100,		/* %0000000000000001	* 1/16 */
	0x0101,		/* %0000000100000001	* 2/16 */
	0x0842,		/* %0000100001000010	* 3/16 */
	0x1111,		/* %0001000100010001	* 4/16 */
	0x2249,		/* %0010001001001001	* 5/16 */
	0x4929,		/* %0100100100101001	* 6/16 */
	0x52aa,		/* %0101001010101010	* 7/16 */
	0x5555,		/* %0101010101010101	* 8/16 */
	0xaaab,		/* %1010101010101011	* 9/16 */
	0x56ab,		/* %0101011010101011	* 10/16 */
	0xddb6,		/* %1101110110110110	* 11/16 */
	0xeeee,		/* %1110111011101110	* 12/16 */
	0xeeef,		/* %1110111011101111	* 13/16 */
	0xefef,		/* %1110111111101111	* 14/16 */
	0xfeff,		/* %1111111011111111	* 15/16 */
	0xffff		/* %1111111111111111	* 16/16 */
};

void
motor(m, s)
	int	m;	/* motor to operate on */
	int	s;	/* speed to set it to. */
{
	unsigned char motor_mask;

	/* These checks take precious space but save on debugging time */
	if ((m > 4) || (m < 1))
		return;
	if ((s < -16) || (s > 16))
		return;

	/* calculate the bits to set in motor_ctl */
	m--;	/* make m = 0 ... 3 */
	motor_mask = (0x11 << m);
	motor_ctl &= ~motor_mask;  /* clear existing bits */

	/* Turn off the motor (just return, above cleared the bits) */
	if (s == 0)
		return;
	
	/* adjust for negative value */
	if (s < 0) {
		motor_mask &= 0xf0; /* turn off direction bit */
		s = 0 - s;
	}

	/* set the speed and the motor control byte */
	speed[m] = speed_table[s];
	motor_ctl |= motor_mask;
	return;
}
