/*
 * servo.c Servo module.
 *
 * Created 7/23/94 by Chuck McManis.
 *
 * This module contains the code and interrupt handler for a servo
 * driver for the miniboard. Three servos can be driven from OC1,
 * OC2, and OC3
 */

#include <hc11.h>

extern unsigned int servo_position[];

struct servo_param {
	unsigned int	neutral_value;
	unsigned char	step_value;
};

extern struct servo_param parms[];

/*
 * servo()
 *
 * This function positions servo number 's' at position 'p'. 
 * The position is a signed value from -100 to 100 percent.
 * The value '0' is considered the neutral position.
 */
void
servo(s, p)
	int	s;	/* Servo to move */
	int	p;	/* Position for the servo */
{
	int i;

	if ((p < -100) || (p > 100))
		return;
	if ((s < 1) || (s > 3))
		return;
	s--;	/* adjust number to 0 - 2 */
	i = (p * (int)(parms[s].step_value)) + (int)(parms[s].neutral_value);
	servo_position[s] = (unsigned int) i;
	return;
}

/*
 * servo_off(s)
 *
 * This function will turn the requested servo "off". This has the
 * effect of disconnecting the ouput compare function from the associated
 * pin on PORT A.
 */
void
servo_off(s)
	int	s;
{
	if ((s < 0) || (s > 3))
		return;
	servo_position[s-1] = 0;
	switch (s) {
	case 1:
		OC1M = 0;	/* Turn off OC1 */
		break;
	case 2:
		TCTL1 &= ~0xc0;	/* Turn off OC2 */
		break;
	case 3:
		TCTL1 &= ~0x30;	/* Turn off OC3 */
		break;
	}
}

