/*
 * Tonetest.c 
 *
 * Created 7/20/94 by Chuck McManis
 * Modified to play a song on 7/24/94
 *
 * This module plays a passable version of twinkle twinkle little star
 * on the beeper. It suffers from two problems (both inherent to the
 * implementation) one the frequencies are only approximate so if you
 * don't have a lead ear, several notes are out of tune, and second
 * because the beeper can preempt the system clock ticks, the duration
 * is less than accurate. What happens is that the higher the note the
 * more often a 'tick' pulse is missed so the longer the duration.
 * Re coding how the system computes clocks would probably fix this.
 * You can also adjust the duratio value based on what frequency your
 * playing. The effect is linear so is easily calculated.
 *
 * As an example though, this shows how initialized data gets stored
 * in the data segment which is in EEPROM. If you have your compile
 * switches set correctly (ie you relocate text, bss, and reset_vectors
 * only) the initialized data below will appear in the text segment after
 * the program. Now while it is useful in our example, you cannot actually
 * _change_ the values in the song[] array or the chromatic[] array. This
 * means that if you had a statement:
 *	n->dur = 600;
 *	if (n->dur == 600) ... do something ...
 *
 * The compiler would accept it and compile it and down load it, however
 * when you tried to execute it the value in the note structure would _not_
 * change and the if statement would not execute.
 */

#define WANT_BEEPER
#include <mboard.h>
#include <mbintsvc.h>

const unsigned int chromatic[] = {
	    /*	 C    D    E    F    G    A    B    */
		262, 294, 330, 349, 392, 440, 494,
		523, 587, 659, 698, 784, 880, 988 };
#define C0 0
#define D0 1
#define E0 2
#define F0 3
#define G0 4
#define A0 5
#define B0 6
#define C1 7
#define D1 8
#define E1 9
#define F1 10
#define G1 11
#define A1 12
#define B1 13

struct note {
	unsigned int	note_val,	/* note */
			dur;		/* duration */
	} song[] = {
	{ G0, 200 }, { G0, 200 }, { D1, 200 }, { D1, 200 },
	{ E1, 200 }, { E1, 200 }, { D1, 400 },
	{ C1, 200 }, { C1, 200 }, { B0, 200 }, { B0, 200 },
	{ A0, 200 }, { A0, 200 }, { G0, 400 },
	{ D1, 200 }, { D1, 200 }, { C1, 200 }, { C1, 200 },
	{ B0, 200 }, { B0, 200 }, { A0, 400 },
	{ D1, 200 }, { D1, 200 }, { C1, 200 }, { C1, 200 },
	{ B0, 200 }, { B0, 200 }, { A0, 400 },
	{ G0, 200 }, { G0, 200 }, { D1, 200 }, { D1, 200 },
	{ E1, 200 }, { E1, 200 }, { D1, 400 },
	{ C1, 200 }, { C1, 200 }, { B0, 200 }, { B0, 200 },
	{ A0, 200 }, { A0, 200 }, { G0, 400 }, { 0, 0}
};

main()
{
	int	i;
	struct note *n;

	puts("Twinkle, Twinkle.\r\n");
	while(1) {
		n = &(song[0]);
		while (n->note_val) {
			tone(chromatic[n->note_val], n->dur);
			msleep(50);
			n++;
		}
		msleep(2500);
	}
 }
