#include <stdlib.h>
#include <hc11.h>
#include "mtask.h"
#include "_task.h"

TASK **_mtask_procs;
TASK *_mtask_task;
char *_mtask_sp;
int _mtask_arg;
void (*_mtask_func)();
void (*InterruptDriver)();

static int gbl_pid;

int create_process(void (*func)(), int arg, int ticks, int stack_size)
	{
	TASK *p;

	INTR_OFF();
	if (stack_size == 0)
		stack_size = DEFAULT_STACK_SIZE;
	else
		stack_size += 20;
	if (ticks == 0)
		ticks = DEFAULT_TICKS;
		
	p = calloc(1, sizeof (TASK) + stack_size - 1);
	p->func = func;
	p->arg = arg;
	p->state = T_CREATED;
	p->pid = ++gbl_pid;
	p->ticks = ticks;
	p->sp = &p->stack[stack_size-1];

	if (_mtask_procs == 0)
		_mtask_procs = &p->next, p->next = p;
	else
		p->next = *_mtask_procs, *_mtask_procs = p;
	INTR_ON();
	return p->pid;
	}

void defer(void)
	{
	INTR_OFF();
	_mtask_sp = (char *)&_mtask_task->sp;
	asm("	pshy\n"
		"	pshx\n"
		"	pshb\n\tpsha\n"
		"	tpa\n\toraa #0x10\n\tpsha\n"
		"	ldy %_mtask_sp\n"
		"	sts 0,y\n"
		"	jmp __mtask_schedule");
	}

void _mtask_null_task(int x)
	{
	while (1)
		defer();
	}
	
void start_scheduler(void)
	{
	create_process(_mtask_null_task, 42, 0, 5);
	/* TOC 4 */
	TMSK1 |= 0x10;
	TFLG1 |= 0x10;
	TCTL1 &= 0xF3;	/* no pin function */
	_mtask_schedule();
	}
	
void _mtask_check_task(void)
	{
	TFLG1 |= 0x10;
	TOC4 += 2000;
	if (InterruptDriver)
		(*InterruptDriver)();
	if (--_mtask_task->current_ticks == 0)
		{
		_mtask_task->sp = _mtask_sp;
		_mtask_schedule();
		}
	}
	
void _mtask_schedule()
	{

	_mtask_task = *_mtask_procs;
	_mtask_procs = &(_mtask_task->next);
	if (_mtask_task->state == T_RUNNING)
		{
		_mtask_task->current_ticks = _mtask_task->ticks;
		_mtask_sp = _mtask_task->sp;
		asm("	lds %_mtask_sp\n"
			"	rti");
		}
	else /* if (task->state == T_CREATED) */
		{
		_mtask_task->state = T_RUNNING;
		_mtask_task->current_ticks = _mtask_task->ticks;
		_mtask_sp = _mtask_task->sp;
		_mtask_func = _mtask_task->func;
		_mtask_arg = _mtask_task->arg;
		asm("	lds	%_mtask_sp\n"
			"	ldd #__mtask_schedule\n"
			"	pshb\n\tpsha\n"
			"	ldd %_mtask_func\n"
			"	pshb\n\tpsha");
		asm("	pshy\n\tpshx\n"
			"	ldd %_mtask_arg\n"
			"	psha\n\tpshb\n"
			"	anda #0xEF\n\tpsha\n"
			"	rti");
		}
	}
