;	Copyright (C) 1995 by ImageCraft, Christina J. Willrich
;	permission to use / modify and distribute from Gordon Doughman

;******************************************************************************
;                                  HC11FP                                     *
;                                                                             *
;                              Copyright 1986                                 *
;                                    by                                       *
;                              Gordon Doughman                                *
;                                                                             *
;       The source code for this floating point package for the MC68HC11      *
;       may be freely distributed under the rules of public domain. However   *
;       it is a copyrighted work and as such may not be sold as a product     *
;       or be included as part of a product for sale without the express      *
;       permission of the author. Any object code produced by the source      *
;       code may be included as part of a product for sale.                   *
;                                                                             *
;       If there are any questions or comments about the floating point       *
;       package please feel free to contact me.                               *
;                                                                             *
;                              Gordon Doughman                                *
;                              Motorola Semiconductor                         *
;                              3490 South Dixie Drive                         *
;                              Dayton, OH  45439                              *
;                              (513) 294-2231                                 *
;                                                                             *
;******************************************************************************
;                                                                             *
;                                 MATH11                                      *
;                                                                             *
;                  Revisions to FP11 floating point code                      *
;                  Modifications Copyright 1988, Scott Wagner                 *
;                                                                             *
;        The following improvements have been made to the HC11FP code:        *
;               1) Execution time and stack space requirements of the basic   *
;                  arithmetic operations (+-*/) have been reduced.            *
;               2) The efficiency of the trigonometric functions has been     *
;                  improved.                                                  *
;               3) New functions FLTATAN, FLTLN, FLTLGT, FLTETOX, FLT10TX     *
;                  and FLTXTOY have been added.                               *
;               4) The polynomial expansion routine POLYNOM is available to   *
;                  evaluate series of arbitrary order.  The coefficient table *
;                  supplied by the user determines the polynomial order.      *
;               5) All floating point numbers are now stored in memory in the *
;                  IEEE floating point format for compatibility with other    *
;                  compilers.                                                 *
;               6) All functions now signal errors by setting the Carry bit.  *
;                  If Carry is clear on return, contents of ACCA are          *
;                  indeterminate; if Carry is set, error code is in ACCA.     *
;               7) For compatibility with error returns, FLTCMP returns Z=1   *
;                  if FPACC1 == FPACC2, Z=0 if FPACC1 != FPACC2, and N=1 if   *
;                  FPACC1 < FPACC2, N=0 if FPACC1 >= FPACC2.  Note that this  *
;                  changes the conditional branch instructions following      *
;                  calls to FLTCMP in the routines in this package which use  *
;                  the FLTCMP routine.                                        *
;                                                                             *
;        These modifications to the HC11FP package are provided under the     *
;        rules of public domain stated above.  Please direct comments about   *
;        these modifications to:                                              *
;                                       Scott Wagner                          *
;                                       Rochester Instrument Systems          *
;                                       255 North Union Street                *
;                                       Rochester, New York 14605             *
;                                                                             *
;******************************************************************************
;
;               REVISION HISTORY:
;       1.0     02-11-88        Release to public domain of enhancements to
;                               Gordon Doughman's 68HC11 FP package.  S. Wagner
;       1.1     07-07-88        Corrected errors/anomalies in FLTCMP.  Made
;                               error returns more consistent.  S. Wagner
;       1.2     07-22-88        Corrected two digit exponent pointer error and
;                               mantissa conversion overflow error in ASCFLT.
;                               Addition involving zero or arguments with large
;                               exponent difference now returns no error and
;                               ACCA clear.  ASCFLT accepts 'E' or 'e' as
;                               exponent delimiter.  S. Wagner
;       1.3     09-21-88        Corrected error in storage format - storage is
;                               now true IEEE floating point format. Added new
;                               functions FLTABS (absolute value), FLTSGN
;                               (signum function), FLTMIN (negative). S. Wagner
;
;
;         org    0x0000
;
          .area    bss
__fpacc1_exp:: 			.blkb    1	; FLOATING POINT ACCUMULATOR #1.
__fpacc1_mantissa:: 	.blkb    3
__fpacc1_mantissa_sign::.blkb    1	; MANTISSA SIGN FOR FPACC1 (0=+, FF=-).
__fpacc2_exp:: 			.blkb    1	; FLOATING POINT ACCUMULATOR #2.
__fpacc2_mantissa::		.blkb    3
__fpacc2_mantissa_sign::.blkb    1	; MANTISSA SIGN FOR FPACC2 (0=+, FF=-).

;
;
	.globl	OVFERR,UNFERR,DIV0ERR,TOLGSMER,NSQRTERR
	.globl	TAN90ERR,LNNEGERR,ACOSERR,ASINERR,FLTFMTER
OVFERR    =    1          ; /* floating point overflow error */
UNFERR    =    1          ; /* floating point underflow error */
DIV0ERR   =    5          ; /* division by 0 error */
TOLGSMER  =    6          ; /* number too large or small to convert to int. */
NSQRTERR  =    7          ; /* tried to take the square root of negative # */
TAN90ERR  =    8          ; /* TANgent of 90 degrees attempted */
LNNEGERR  =    9          ; /* LOG or LN of negative number or 0 */
ACOSERR   =    10         ; /* Arc cosine not implemented */
ASINERR   =    11         ; /* Arc sine not implemented */
FLTFMTER  =    12         ; /* floating point format error in ASCFLT */
;
	.area	text
__fp_is_acc1_zero::          ;    CHECKS FOR ZERO IN FPACC POINTED TO BY X.
         pshb                ;SAVE D.
         psha
         ldd    0,x          ;GET FPACC EXPONENT & HIGH 8 BITS.
         bne    CHCK1        ;NOT ZERO. RETURN.
         ldd    2,x          ;CHECK LOWER 16 BITS.
CHCK1:   pula               ; RESTORE D.
         pulb
         rts                 ;RETURN WITH CC SET.
;
;
__fp_xfer_acc1_to_acc2::
         ldd    __fpacc1_exp     ;GET FPACC1 EXPONENT & HIGH 8 BIT OF MANTISSA.
         std    __fpacc2_exp     ;PUT IT IN FPACC2.
         ldd    __fpacc1_mantissa+1   ;GET FPACC1 LOW 16 BITS OF MANTISSA.
         std    __fpacc2_mantissa+1   ;PUT IT IN FPACC2.
         ldaa   __fpacc1_mantissa_sign     ;TRANSFER THE SIGN.
         staa   __fpacc2_mantissa_sign
         rts                 ;RETURN.
;
;*****************************************************************************
;                                                                            *
;        The following two subroutines, __push_fpacc2 & PULPFAC2, push FPACC2     *
;        onto and pull FPACC2 off of the hardware stack respectively.        *
;        The number is stored in the "memory format".                        *
;                                                                            *
;*****************************************************************************
;
;
__push_fpacc2::
         pulx               ; GET THE RETURN ADDRESS OFF OF THE STACK.
         pshx               ; ALLOCATE FOUR BYTES OF STACK SPACE.
         pshx
         xgdx               ; PUT THE RETURN ADDRESS IN D.
         tsx                ; POINT TO THE STORAGE AREA.
         pshb               ; PUT THE RETURN ADDRESS BACK ON THE STACK.
         psha
         jmp    __from_fpacc2    ; GO PUT FPACC2 ON THE STACK & RETURN.
;
;
__pull_fpacc2::
         tsx                ; POINT TO THE RETURN ADDRESS.
         inx                ; POINT TO THE SAVED NUMBER.
         inx
         jsr    __to_fpacc2    ; RESTORE FPACC2.
         pulx               ; GET THE RETURN ADDRESS OFF THE STACK.
         ins                ; REMOVE THE NUMBER FROM THE STACK.
         ins
         ins
         ins
         jmp    0,x         ; RETURN.
;
;
;
;*****************************************************************************
;                                                                            *
;                           GETFPACx SUBROUTINE                              *
;                                                                            *
;       The __to_fpacc1 and __to_fpacc2 subroutines get a floating point number    *
;       stored in memory and put it into either FPACC1 or FPACC2 in a format *
;       that is expected by all the floating point math routines. These      *
;       routines convert the IEEE binary floating point format to the format *
;       required by the math routines.  The IEEE format converted by these   *
;       routines is shown below:                                             *
;                                                                            *
;       31 30_______23 22_____________________0                              *
;       s   exponent          mantissa                                       *
;                                                                            *
;       The exponent is biased by 127 to facilitate floating point           *
;       comparisons.  The sign bit is 0 for positive numbers and 1           *
;       for negative numbers.  The mantissa is stored in hidden bit          *
;       normalized format so that 24 bits of precision can be obtained.      *
;       Since a normalized floating point number always has its most         *
;       significant bit set, we can use the 24th bit to hold the exponent    *
;       LSB.  This allows us to get 24 bits of precision in the mantissa     *
;       and store the entire number in just 4 bytes.  The format required by *
;       the math routines uses a seperate byte for the sign, therfore each   *
;       floating point accumulator requires five bytes.                      *
;                                                                            *
;*****************************************************************************
;
;
RETONE:   ldx    #ONE       ;  POINT TO CONSTANT 1.0
__to_fpacc1::
         clr    __fpacc1_mantissa_sign    ; SET UP FOR POSITIVE NUMBER.
         ldd    2,x         ; GET LOW 16-BITS OF THE MANTISSA.
         std    __fpacc1_mantissa+1  ; PUT IN FPACC1.
         ldd    0,x         ; GET THE EXPONENT & HIGH BYTE OF THE MANTISSA
         lsld               ; SHIFT SIGN INTO CARRY; EXPONENT INTO ACCA
         bcc    GETFP11     ; IF NUMBER IS POSITIVE, SKIP SETTING THE SIGN BYTE
         com    __fpacc1_mantissa_sign    ; SET SIGN TO NEGATIVE.
GETFP11:  staa   __fpacc1_exp   ;  STORE EXPONENT; CHECK FOR ZERO
         beq    GETFP12     ; IF NUMBER IS ZERO, DON'T SET MANTISSA MSB
         sec                ; SET CARRY TO SHIFT INTO MANTISSA MSB
         rorb               ; NORMALIZED MANTISSA NOW IN B
GETFP12:  stab   __fpacc1_mantissa   ;  PUT IN FPACC1.
         clrb               ; NO ERRORS.
         rts                ; RETURN.
;
;
__to_fpacc2::
         clr    __fpacc2_mantissa_sign    ; SET UP FOR POSITIVE NUMBER.
         ldd    2,x         ; GET LOW 16-BITS OF THE MANTISSA.
         std    __fpacc2_mantissa+1  ; PUT IN FPACC2.
         ldd    0,x         ; GET THE EXPONENT & HIGH BYTE OF THE MANTISSA
         lsld               ; SHIFT SIGN INTO CARRY; EXPONENT INTO ACCA
         bcc    GETFP21     ; IF NUMBER IS POSITIVE, SKIP SETTING THE SIGN BYTE
         com    __fpacc2_mantissa_sign    ; SET SIGN TO NEGATIVE.
GETFP21:  staa   __fpacc2_exp   ;  STORE EXPONENT; CHECK FOR ZERO
         beq    GETFP22     ; IF NUMBER IS ZERO, DON'T SET MANTISSA MSB
         sec                ; SET CARRY TO SHIFT INTO MANTISSA MSB
         rorb               ; NORMALIZED MANTISSA NOW IN B
GETFP22:  stab   __fpacc2_mantissa   ;  PUT IN FPACC2.
         rts                ; RETURN.
;
;
;
;*****************************************************************************
;                                                                            *
;                        PUTFPACx SUBROUTINE                                 *
;                                                                            *
;       These two subroutines perform to opposite function of __to_fpacc1 and   *
;       __to_fpacc2. Again, these routines are used to convert from the         *
;       internal format used by the floating point package to the IEEE       *
;       floating point format. See the __to_fpacc1 and __to_fpacc2, documentation  *
;       for a description of the IEEE format.                                *
;                                                                            *
;*****************************************************************************
;
;
__from_fpacc1::
         ldd    __fpacc1_mantissa+1  ; GET L.S. 16 BITS OF THE MANTISSA.
         std    2,x         ; SAVE IT
         ldd    __fpacc1_exp    ; GET FPACC1 EXPONENT & UPPER 8 BITS OF MANT.
         lslb               ; DROP MANTISSA MSB (IMPLIED), ALSO MAKE ACCB < 0xFF
         cmpb   __fpacc1_mantissa_sign    ; SIGN BIT INTO CARRY. (B-0xFF => C SET; B-0 =>C CLR)
         rora               ; NOW ACCA HAS SIGN:EXPONENT[7-1]; EXPONENT[0] => C
         rorb               ; NOW ACCB HAS EXPONENT[0]:MANTISSA[22-16]
         std    0,x         ; SAVE IT IN MEMORY
         rts
;
;
__from_fpacc2::
         ldd    __fpacc2_mantissa+1  ; GET L.S. 16 BITS OF THE MANTISSA.
         std    2,x         ; SAVE IT
         ldd    __fpacc2_exp    ; GET FPACC2 EXPONENT & UPPER 8 BITS OF MANT.
         lslb               ; DROP MANTISSA MSB (IMPLIED), ALSO MAKE ACCB < 0xFF
         cmpb   __fpacc2_mantissa_sign    ; SIGN BIT INTO CARRY. (B-0xFF => C SET; B-0 =>C CLR)
         rora               ; NOW ACCA HAS SIGN:EXPONENT[7-1]; EXPONENT[0] => C
         rorb               ; NOW ACCB HAS EXPONENT[0]:MANTISSA[22-16]
         std    0,x         ; SAVE IT IN MEMORY
         rts

__copy_float::
	ldd		0,y
	std		0,x
	ldd		2,y
	std		2,x
	rts

ONE::    .byte    0x3F,0x80,0x00,0x00 
CONSTP5::.byte    0x3F,0x00,0x00,0x00        ; +(1/2!)
CONSTP1::  .byte    0x3D,0xCC,0xCC,0xCD   ;      0.1 DECIMAL
CONST10::  .byte    0x41,0x20,0x00,0x00   ;      10.0 DECIMAL

__fpneg::
         tst    __fpacc1_mantissa    ; CHECK FOR ZERO
         beq    FLTMINZ     ; DO NOTHING IF ZERO
         com    __fpacc1_mantissa_sign    ; CHANGE SIGN
FLTMINZ:  clrb              ;  CONDITION CODE 0
         rts
LNTBL::    ;
         .byte    0x3E,0x1D,0x89,0xD9       ;  2/13
         .byte    0x3E,0x3A,0x2E,0x8C       ;  2/11
         .byte    0x3E,0x63,0x8E,0x39       ;  2/9
         .byte    0x3E,0x92,0x49,0x25       ;  2/7
         .byte    0x3E,0xCC,0xCC,0xCD       ;  2/5
         .byte    0x3F,0x2A,0xAA,0xAB       ;  2/3
         .byte    0x40,0x00,0x00,0x00       ;  2/1
         .byte    0xFF                    ; END OF TABLE
;
MONE::     .byte    0xBF,0x80,0x00,0x00     ;    -1.0
;
NLN2::     .byte    0x3F,0x31,0x72,0x18      ;   LN(2)
;
N1DLN10::  .byte    0x3E,0xDE,0x5B,0xD9      ;   1/LN(10)
