	.area	text
;*****************************************************************************
;                                                                            *
;                   FLOATING POINT TO INTEGER CONVERSION                     *
;                                                                            *
;        The subroutine FLT2INT will perform  floating point to integer      *
;        conversion.  The floating point number if positive, will be         *
;        converted to an unsigned 16 bit integer ( 0 <= X <= 65535 ), and    *
;        the N flag will be cleared.  If negative, the floating point number *
;        will be converted to a signed 16-bit (twos complement) integer      *
;        (-32768 <= X <= -1), and the N flag will be set.  The converted     *
;        integer is returned in the double accumulator D.  Truncation is     *
;        always toward zero, and the fractional part of the argument is      *
;        discarded.  If the argument is too large or too small, the carry    *
;        flag is set and the error code is returned in ACCA.                 *
;                                                                            *
;        The subroutine FLTROUND is similar to FLT2INT except that it rounds *
;        to the nearest integer instead of truncating the fractional part.   *
;                                                                            *
;*****************************************************************************
;
;
;
FLTROUND: ;
         jsr    __push_fpacc2            ; SAVE FPACC2
         ldx    #CONSTP5            ; POINT TO CONSTANT 0.5
         jsr    __to_fpacc2            ; PUT IT INTO FPACC2
         tst    __fpacc1_mantissa_sign            ; CHECK FOR NEGATIVE ARGUMENT
         beq    FLTROU1             ; ARGUMENT POSITIVE - ADD +0.5
         com    __fpacc2_mantissa_sign            ; ARGUMENT NEGATIVE - ADD -0.5
FLTROU1:  jsr    __fpadd             ;  ADD 0.5 TO ROUND
         jsr    __pull_fpacc2            ; RESTORE FPACC2 - FALL THROUGH TO FLT2INT
;
__fp_to_int::
         clra                ;ZERO UPPER BYTE OF D
         ldab   __fpacc1_exp     ;GET EXPONENT
         cmpb   #0x7F        ; CHECK FOR INTEGRAL PART
         bhs    FLT2INT1     ;IF INTEGRAL PART
         clrb                ;NO INTEGRAL PART
         bra    FLT2INT4     ;RETURN ZERO
FLT2INT1: comb               ; COMPUTE NUMBER OF SHIFTS REQUIRED ...
         addd   #0xFF90      ; ... TO GENERATE RESULT
         ble    FLT2INTE     ;ERROR IF > 65535
         xgdx                ;SAVE SHIFT COUNTER IN X
         ldd    __fpacc1_mantissa     ;GET SIGNIFICANT PART OF MANTISSA
         bra    FLT2INT3     ;GO DO SHIFTING
FLT2INT2: lsrd               ; SHIFT RESULT
FLT2INT3: dex                ; DECREMENT SHIFT COUNTER
         bne    FLT2INT2     ;IF NOT DONE SHIFTING YET
         tst    __fpacc1_mantissa_sign     ;CHECK FOR POSITIVE ARGUMENT
         bpl    FLT2INT4     ;IF POSITIVE, WE ARE DONE
         coma                ;COMPLEMENT RESULT
         comb
         addd   #1           ;ADD 1 FOR TWOS COMPLEMENT
         bpl    FLT2INTE     ;IF NEGATIVE RESULT < -32768
FLT2INT4: clc                ; NO ERRORS
         rts                 ;RETURN
FLT2INTE: ldaa   #TOLGSMER   ; NUMBER TOO LARGE OR TOO SMALL
         sec                 ;FLAG ERROR
         rts                 ;RETURN
