;*****************************************************************************
;                                                                            *
;                       FLOATING POINT ADDITION                              *
;                                                                            *
;       This subroutine performs floating point addition of the two numbers  *
;       in FPACC1 and FPACC2.  The result of the addition is placed in       *
;       FPACC1 while FPACC2 remains unchanged.  This subroutine performs     *
;       full signed addition so either number may be of the same or opposite *
;       sign.                                                                *
;                          WORSE CASE = 563 CYCLES = 282 uS @ 2MHz           *
;                                                                            *
;*****************************************************************************
;
;
	.area	text
__fpadd::
         ldaa   __fpacc1_exp     ;LOAD FP1 EXPONENT
         bne    FLTADD1      ;IF NOT ZERO
TFR2TO1: ldd    __fpacc2_exp    ; GET FPACC2 EXPONENT & HIGH 8 BIT OF MANTISSA.
         std    __fpacc1_exp     ;PUT IT IN FPACC1.
         ldd    __fpacc2_mantissa+1   ;GET FPACC2 LOW 16 BITS OF MANTISSA.
         std    __fpacc1_mantissa+1   ;PUT IT IN FPACC1.
         ldaa   __fpacc2_mantissa_sign     ;TRANSFER THE SIGN.
         staa   __fpacc1_mantissa_sign
FLTADDR:  clrb               ; NO ERRORS.
         rts                 ;RETURN.
FLTADD1:  tst    __fpacc2_exp    ; CHECK FP2 EXPONENT
         beq    FLTADDR      ;RETURN IF ZERO
         ldab   __fpacc1_mantissa_sign     ;LOAD SIGN FROM 1
         eorb   __fpacc2_mantissa_sign     ;DECIDE TO ADD OR SUBTRACT
         suba   __fpacc2_exp     ;COMPARE EXPONENTS
         bcs    FLTADD2      ;IF FPACC2 > FPACC1
         cmpa   #24          ;IF FPACC1 >> FPACC2
         bhi    FLTADDR      ;RETURN UNDERFLOW
         pshy                ;SAVE Y REGISTER
         psha                ;SAVE SHIFT COUNTER
         ldx    #__fpacc2_mantissa    
		 					;ADDEND TO BE DENORMALIZED POINTED TO BY X
         ldy    #__fpacc1_mantissa    
		 					;ADDEND TO BE LEFT NORMALIZED POINTED TO BY Y
         bra    FLTADD3      ;GO DO NORMALIZATION
FLTADD2:  nega               ; CHANGE SIGN OF A FOR SHIFT COUNTER
         cmpa   #24          ;IF FPACC2 >> FPACC1
         bhi    TFR2TO1      ;PUT FPACC2 INTO FPACC1 AND RETURN
         pshy                ;SAVE Y REGISTER
         psha                ;SAVE SHIFT COUNTER
         ldaa   __fpacc2_exp     ;LOAD EXPONENT FROM 2
         staa   __fpacc1_exp     ;... AND TRANSFER IT TO 1
         ldaa   __fpacc2_mantissa_sign     ;LOAD SIGN FROM 2
         staa   __fpacc1_mantissa_sign     ;... AND TRANSFER IT TO 1
         ldx    #__fpacc1_mantissa    
		 					;ADDEND TO BE DENORMALIZED POINTED TO BY X
         ldy    #__fpacc2_mantissa    
		 					;ADDEND TO BE LEFT NORMALIZED POINTED TO BY Y
FLTADD3:  tba                ; PUT ADD/SUBTRACT FLAG BYTE IN BOTH A AND B
         psha                ;CREATE 3 BYTE STACK WORKSPACE SET TO 0x00 FOR ...
         psha                ;... ADDITION AND 0xFF FOR SUBTRACTION.
         psha
         eora   0,x          ;LOAD MANTISSA MSB; COMPLEMENT IF SUBTRACTION
         psha                ;PUT IN STACK WORKSPACE
         tba                 ;PUT ADD/SUBTRACT FLAG INTO A AGAIN
         eora   1,x          ;LOAD MANTISSA NSB; COMPLEMENT IF SUBTRACTION
         psha                ;PUT IN STACK WORKSPACE
         tba                 ;PUT ADD/SUBTRACT FLAG INTO A AGAIN
         eora   2,x          ;LOAD MANTISSA LSB; COMPLEMENT IF SUBTRACTION
         psha                ;PUT IN STACK WORKSPACE
         tsx                 ;PUT POINTER TO WORKSPACE IN X
         ldaa   6,x          ;GET SHIFT COUNTER
         stab   6,x          ;SAVE ADD/SUBTRACT FLAG
         tab                 ;B REGISTER WILL BE BYTE DISPLACEMENT COUNTER
         anda   #0x07        ; REMOVE BYTE DISPLACEMENT FROM A
         inca                ;PRE-INCREMENT SHIFT COUNTER
         andb   #0x18        ; REMOVE SHIFT DISPLACEMENT FROM B
         lsrb                ;RIGHT JUSTIFY BYTE DISPLACEMENT COUNTER
         lsrb
         lsrb
         abx                 ;DO BYTE DISPLACEMENT BY ADDING TO POINTER
         ldab   3,x          ;LOAD ADD/SUBTRACT FLAG (OK SINCE FLAG IN 4 BYTES)
FLTADD4:  deca               ; DECREMENT BYTE SHIFT COUNTER
         beq    FLTADD5      ;IF DONE SHIFTING
         rorb                ;SHIFT ADD/SUBTRACT FLAG INTO CARRY
         ror    2,x          ;SHIFT MANTISSA MSB
         ror    1,x          ;SHIFT MANTISSA NSB
         ror    0,x          ;SHIFT MANTISSA LSB
         bra    FLTADD4      ;GO AROUND AGAIN
FLTADD5:  rorb               ; SHIFT ADD/SUBTRACT FLAG INTO CARRY
         ldaa   2,y          ;LOAD NORMALIZED ADDEND LSB
         adca   0,x          ;ADD IN DENORMALIZED ADDEND LSB
         staa   __fpacc1_mantissa+2   ;STORE NORMALIZED SUM LSB
         ldaa   1,y          ;LOAD NORMALIZED ADDEND NSB
         adca   1,x          ;ADD IN DENORMALIZED ADDEND NSB
         staa   __fpacc1_mantissa+1   ;STORE NORMALIZED SUM NSB
         ldaa   0,y          ;LOAD NORMALIZED ADDEND NSB
         adca   2,x          ;ADD IN DENORMALIZED ADDEND NSB
         staa   __fpacc1_mantissa     ;STORE NORMALIZED SUM MSB
         tsx                 ;RESTORE WORKSPACE POINTER
         ldaa   6,x          ;GET BACK ADD/SUBTRACT FLAG
         bmi    FLTADD6      ;OPERATION WAS SUBTRACTION
         bcc    FLTADD7      ;IF ADD AND NO CARRY, DO NOTHING
         ror    __fpacc1_mantissa     ;CARRY WAS SET; ROTATE TO NORMALIZE OVERFLOW
         ror    __fpacc1_mantissa+1
         ror    __fpacc1_mantissa+2
         inc    __fpacc1_exp     ;NOW INCREMENT EXPONENT TO ACCOMODATE SHIFT
         bne    FLTADD7      ;SUCCESSFUL COMPLETION
         ldd    #__fp_return_max      ;OVERFLOW - RETURN MAXIMUM NUMBER
         bra    FLTADD11
FLTADD6:  bcs    FLTADD8     ; NO SIGN CHANGE
         com    __fpacc1_mantissa_sign     ;CHANGE SIGN OF RESULT
         neg    __fpacc1_mantissa+2   ;CHANGE SIGN OF LSB
         bcc    FLTADD12     ;IF LSB WAS ZERO
         inc    __fpacc1_mantissa+1   ;CARRY FROM LSB
         bne    FLTADD12     ;IF NSB NOT ZERO
         inc    __fpacc1_mantissa     ;CARRY FROM NSB
FLTADD12: neg    __fpacc1_mantissa+1  ; CHANGE SIGN OF NSB
         bcc    FLTADD13     ;IF NSB WAS ZERO
         inc    __fpacc1_mantissa     ;CARRY FROM NSB
FLTADD13: neg    __fpacc1_mantissa    ; CHANGE SIGN OF MSB
FLTADD8:  tst    __fpacc1_mantissa    ; IS MANTISSA NORMALIZED?
         bmi    FLTADD7      ;YES - DONE
         bne    FLTADD9      ;DO BIT SHIFT
         ldaa   __fpacc1_exp     ;DECREMENT EXPONENT BY 8 FOR BYTE SHIFT
         suba   #8
         bls    FLTADD10     ;IF UNDERFLOW
         staa   __fpacc1_exp     ;REPLACE DECREMENTED EXPONENT
         ldd    __fpacc1_mantissa+1   ;DO BYTE SHIFT
         beq    FLTADD10     ;IF RESULT MANTISSA IS ZERO
         clr    __fpacc1_mantissa+2   ;CLEAR BYTE 3
         std    __fpacc1_mantissa     ;STORE BYTES 1 AND 2
         bra    FLTADD8
FLTADD9:  dec    __fpacc1_exp    ; DECREMENT EXPONENT TO ACCOMODATE SHIFT
         beq    FLTADD10     ;IF UNDERFLOW
         lsl    __fpacc1_mantissa+2   ;DO BIT SHIFT
         rol    __fpacc1_mantissa+1
         rol    __fpacc1_mantissa
         bpl    FLTADD9
FLTADD7:  ldy    7,x         ; PULL Y FROM STACK
         ldab   #9           ;RESTORE STACK - ADD 7 TO X (STACK BASE)
         abx                 ;DO ADDITION
         txs                 ;NEW STACK BASE (EFFECTIVELY PULLED STACK)
         clrb                ;NO ERRORS
         rts                 ;RETURN
;
FLTADD10: ldd    #__fp_return_zero    ; UNDERFLOW - RETURN ZERO
FLTADD11: ldy    7,x         ; PULL Y FROM STACK
         std    7,x          ;PUT RETURN ADDRESS ON STACK
         ldab   #7           ;RESTORE STACK - ADD 5 TO X (STACK BASE)
         abx                 ;DO ADDITION
         txs                 ;NEW STACK BASE (EFFECTIVELY PULLED STACK)
         rts                 ;RETURN THROUGH OVERFLOW OR UNDERFLOW
