;*****************************************************************************
;                                                                            *
;                FLOATING POINT COMPARE SUBROUTINE                           *
;                                                                            *
;        This subroutine performs floating point comparison of the arguments *
;        in FPACC1 and FPACC2.  The routine returns Z = 1 if FPACC1 = FPACC2 *
;        and Z = 0 otherwise; N = 1 if FPACC1 < FPACC2 and N = 0 otherwise;  *
;        C = 0, and V = 0 in the condition code register.  The results of    *
;        this comparison may be tested with the twos complement signed       *
;        number conditional branch instructions BLE, BLT, BEQ, BNE, BGT, and *
;        BGE.  FPACC1 and FPACC2 will remain unchanged.                      *
;                                                                            *
;*****************************************************************************
;
;
	.area	text
__fpcmp::
         ldaa   __fpacc2_mantissa_sign     ;IS FPACC2 NEGATIVE?
         bpl    FLTCMP2      ;NO. CONTINUE WITH COMPARE.
         ldab   __fpacc1_mantissa_sign     ;IS FPACC1 NEGATIVE?
         bpl    FLTCMP2      ;NO. CONTINUE WITH COMPARE.
         ldd    __fpacc2_exp     ;YES. BOTH ARE NEGATIVE SO COMPARE MUST BE DONE
         cpd    __fpacc1_exp     ;BACKWARDS. ARE THEY EQUAL SO FAR?
         bne    FLTCMP1      ;NO. RETURN WITH CONDITION CODES SET.
         ldd    __fpacc2_mantissa+1   ;YES. COMPARE LOWER 16 BITS OF MANTISSAS.
         cpd    __fpacc1_mantissa+1
         bra    FLTCMP1
FLTCMP2:  cmpa   __fpacc1_mantissa_sign    ; BOTH POSITIVE?
         bne    FLTCMP1      ;NO. RETURN WITH CONDITION CODES SET.
         ldd    __fpacc1_exp     ;GET FPACC1 EXPONENT & UPPER 8 BITS OF MANTISSA.
         cpd    __fpacc2_exp     ;SAME AS FPACC2?
         bne    FLTCMP1      ;NO. RETURN WITH CONDITION CODES SET.
         ldd    __fpacc1_mantissa+1   ;GET FPACC1 LOWER 16 BITS OF MANTISSA.
         cpd    __fpacc2_mantissa+1   ;COMPARE WITH FPACC2 LOWER 16 BITS OF MANTISSA.
FLTCMP1:  tpa                ; GET CONDITION CODES SO WE CAN MANIPULATE BITS
         anda   #0xF4        ; CLEAR OVERFLOW, NEGATIVE, AND CARRY BITS
         bcc    FLTCMP3      ;IF NO CARRY
         oraa   #0x08        ; SET NEGATIVE BIT
FLTCMP3:  tap                ; PUT CONDITION CODES BACK
         rts                 ;RETURN WITH CONDITION CODES SET.
