;*****************************************************************************
;                                                                            *
;                         FLOATING POINT DIVIDE                              *
;                                                                            *
;        This subroutine performs signed floating point divide. The          *
;        operation performed is FPACC1/FPACC2.  The divisor (FPACC2) is left *
;        unaltered and the answer is placed in FPACC1.  There are several    *
;        error conditions that can be returned by this routine.  They are:   *
;        a) division by zero.  b) overflow.  c) underflow.  As with all      *
;        other routines, an error is indicated by the carry being set and    *
;        the error code being in the A-reg.                                  *
;                                                                            *
;                          WORSE CASE = 495 CYCLES = 248 uS @ 2MHz           *
;                                                                            *
;*****************************************************************************
;
;
	.area	text
__fpdiv::
         ldaa   __fpacc2_mantissa_sign     ;GET FPACC2 MANTISSA SIGN.
         eora   __fpacc1_mantissa_sign     ;SET THE SIGN OF THE RESULT.
         staa   __fpacc1_mantissa_sign     ;SAVE THE RESULT.
         tst    __fpacc2_exp     ;IS THE DIVISOR 0?
         bne    FLTDIV1      ;NO. GO SEE IF THE DIVIDEND IS ZERO.
         ldaa   #DIV0ERR     ;YES. RETURN A DIVIDE BY ZERO ERROR.
         jmp    __fp_return_div0      ;FLAG ERROR AND RETURN.
FLTDIV1:  tst    __fpacc1_exp    ; IS THE DIVIDEND 0?
         bne    FLTDIV2      ;NO. GO PERFORM THE DIVIDE.
         clrb                ;YES. ANSWER IS ZERO. NO ERRORS.
         clr    __fpacc1_mantissa_sign     ;SIGN OF ZERO IS POSITIVE.
         rts                 ;RETURN.
FLTDIV2:  ldaa   __fpacc1_exp    ; NOW COMPUTE EXPONENT - GET NUMERATOR EXPONENT
         suba   __fpacc2_exp     ;SUBTRACT DENOMINATOR EXPONENT
         bhs    FLTDIV8      ;IF CARRY CLEAR, RESULT MUST BE POSITIVE
         bmi    FLTDIV6      ;RESULT IN RANGE
         jmp    __fp_return_zero      ;UNDERFLOW - RETURN ZERO
FLTDIV8:  bmi    FLTDIV7     ; OVERFLOW - RETURN MAX. NUMBER
FLTDIV6:  adda   #0x7E       ;  PUT BACK EXPONENT BIAS
         staa   __fpacc1_exp     ;STORE CORRECTED QUOTIENT EXPONENT
         ldd    __fpacc1_mantissa     
		 				;TO DO DIVIDE, NUMERATOR MANTISSA MUST BE LESS ...
         subd   __fpacc2_mantissa     ;... THAN DENOMINATOR MANTISSA.
         bmi    FLTDIV3      ;BRANCH IF NUMERATOR LESS THAN DENOMINATOR MANTISSA
         bne    FLTDIV4      ;TEST LSBs ONLY IF NUMERATOR = DENOMINATOR
         ldaa   __fpacc1_mantissa+2   ;CHECK LSBs SINCE BYTES 1 AND 2 OF ...
         suba   __fpacc2_mantissa+2   ;NUMERATOR AND DENOMINATOR ARE SAME.
         bmi    FLTDIV3      ;BRANCH IF NUMERATOR LESS THAN DENOMINATOR MANTISSA
FLTDIV4:  ldd    __fpacc1_mantissa    
						; DIVIDE NUMERATOR BY 2 SO IT IS SMALLER THAN DENOM
         lsrd                ;SHIFT TO DIVIDE BYTES 1 AND  2
         ror    __fpacc1_mantissa+2   ;SHIFT BYTE 3
         std    __fpacc1_mantissa     ;PUT BYTES 1 AND 2 BACK
         inc    __fpacc1_exp     ;NOW INCREMENT EXPONENT
         bne    FLTDIV3      ;CHECK FOR OVERFLOW
FLTDIV7:  jmp    __fp_return_max      ; RETURN MAXIMUM MAGNITUDE NUMBER
FLTDIV3:  ldd    __fpacc1_mantissa    ; GET PARTIAL NUMERATOR
         ldx    __fpacc2_mantissa     ;GET PARTIAL DENOMINATOR
         inx                 ;PARTIAL QUOTIENT MUST BE LESS THAN FULL QUOTIENT
         bne    FLTDIV5      ;IF X REGISTER ROLLED OVER, WE BLEW IT ...
         dex                 ;... SO RESTORE X REGISTER TO WHERE IT WAS
FLTDIV5:  fdiv               ; COMPUTE PARTIAL QUOTIENT
         pshy                ;SAVE Y REGISTER
         ldy    __fpacc1_mantissa     ;TEMPORARY STORAGE FOR NUMERATOR
         stx    __fpacc1_mantissa     
		 				;PARTIAL QUOTIENT TO MULTIPLY BY DENOMINATOR
         pshx                ;SAVE PARTIAL QUOTIENT
         ldx    #0           ;SET UP WORK SPACE ON THE STACK.
         pshx
         pshx
         tsx                 ;SET UP POINTER TO WORK SPACE.
         ldaa   __fpacc2_mantissa+2   ;MULTIPLY DENOMINATOR LS BYTE ...
         ldab   __fpacc1_mantissa+1   ;... BY NUMERATOR LS BYTE
         mul
         staa   3,x          ;SAVE PARTIAL RESULT ON STACK
         jsr    __fp_remcomp      
		 					;JUMP INTO FLOATING MULTIPLY TO FINISH OPERATION
         sty    __fpacc1_mantissa     ;RESTORE NUMERATOR
         ldd    __fpacc1_mantissa+1   ;MOVE NUMERATOR BYTES 2 AND 3 TO D
         subd   1,x          ;SUBTRACT TO COMPUTE REMAINDER (ASSUME BYTE 1 SAME)
         ldx    __fpacc2_mantissa     ;LOAD DENOMINATOR
         fdiv                ;DIVIDE REMAINDER BY DENOMINATOR
         stx    __fpacc1_mantissa+1   ;STORE SECOND PARTIAL PRODUCT
         clr    __fpacc1_mantissa     ;MS BYTE OF SECOND PARTIAL PRODUCT IS ZERO
         pulx                ;SCRAP STACK WORKSPACE
         pulx
         pula                ;RETRIEVE FIRST PARTIAL PRODUCT
         pulb
         puly                ;RETRIEVE Y REGISTER
         addd   __fpacc1_mantissa     
		 				;ADD TO SECOND PARTIAL PRODUCT TO GET QUOTIENT
         std    __fpacc1_mantissa     ;SAVE QUOTIENT IN FLOATING ACCUMULATOR
         clrb                ;NO ERRORS
         rts                 ;DONE - RETURN
