;*****************************************************************************
;                                                                            *
;                FLOATING POINT TO ASCII CONVERSION SUBROUTINE               *
;                                                                            *
;        This subroutine performs floating point to ASCII conversion of      *
;        the number in FPACC1.  The ascii string is placed in a buffer       *
;        pointed to by the X index register.  The buffer must be at least    *
;        14 bytes long to contain the ASCII conversion.  The resulting       *
;        ASCII string is terminated by a zero (0) byte.  Upon exit the       *
;        X Index register will be pointing to the first character of the     *
;        string.  FPACC1 and FPACC2 will remain unchanged.                   *
;                                                                            *
;*****************************************************************************
;
;
	.area	text
__fp_to_string::
         pshx                ;SAVE THE POINTER TO THE STRING BUFFER.
         ldx    #__fpacc1_exp    ;POINT TO FPACC1.
         jsr    __fp_is_acc1_zero        ;IS FPACC1 0?
         bne    FLTASC1      ;NO. GO CONVERT THE NUMBER.
         pulx                ;RESTORE POINTER.
         ldd    #0x3000      ; GET ASCII CHARACTER + TERMINATING BYTE.
         std    0,x          ;PUT IT IN THE BUFFER.
         rts                 ;RETURN.
FLTASC1:  ldx    __fpacc1_exp    ; SAVE FPACC1.
         pshx
         ldx    __fpacc1_mantissa+1
         pshx
         ldaa   __fpacc1_mantissa_sign
         psha
         jsr    __push_fpacc2     ;SAVE FPACC2.
         ldx    #0
         pshx                ;ALLOCATE LOCALS.
         pshx
         pshx                ;SAVE SPACE FOR STRING BUFFER POINTER.
         tsy                 ;POINT TO LOCALS.
         ldx    15,y         ;GET POINTER FROM STACK.
         ldaa   #0x20        ; PUT A SPACE IN THE BUFFER IF NUMBER NOT NEGATIVE.
         tst    __fpacc1_mantissa_sign     ;IS IT NEGATIVE?
         beq    FLTASC2      ;NO. GO PUT SPACE.
         clr    __fpacc1_mantissa_sign     ;MAKE NUMBER POSITIVE FOR REST OF CONVERSION.
         ldaa   #'-         ;YES. PUT MINUS SIGN IN BUFFER.
FLTASC2:  staa   0,x
         inx                 ;POINT TO NEXT LOCATION.
         stx    0,y          ;SAVE POINTER.
FLTASC5:  ldx    #N9999999   ; POINT TO CONSTANT 9999999.
         jsr    __to_fpacc2     ;GET INTO FPACC2.
         jsr    __fpcmp       ;COMPARE THE NUMBERS. IS FPACC1 > 9999999?
         bgt    FLTASC3      ;YES. GO DIVIDE FPACC1 BY 10.
         ldx    #P9999999    ;POINT TO CONSTANT 999999.9
         jsr    __to_fpacc2     ;MOVE IT INTO FPACC2.
         jsr    __fpcmp       ;COMPARE NUMBERS. IS FPACC1 > 999999.9?
         bgt    FLTASC4      ;YES. GO CONTINUE THE CONVERSION.
         dec    2,y          ;DECREMENT THE MULT./DIV. COUNT.
         ldx    #CONST10     ;NO. MULTIPLY BY 10. POINT TO CONSTANT.
FLTASC6: jsr    __to_fpacc2    ; MOVE IT INTO FPACC2.
         jsr    __fpmul
         bra    FLTASC5      ;GO DO COMPARE AGAIN.
FLTASC3:  inc    2,y         ; INCREMENT THE MULT./DIV. COUNT.
         ldx    #CONSTP1     ;POINT TO CONSTANT ".1".
         bra    FLTASC6      ;GO DIVIDE FPACC1 BY 10.
FLTASC4:  ldx    #CONSTP5    ; POINT TO CONSTANT OF ".5".
         jsr    __to_fpacc2     ;MOVE IT INTO FPACC2.
         jsr    __fpadd       ;ADD .5 TO NUMBER IN FPACC1 TO ROUND IT.
         ldab   __fpacc1_exp     ;GET FPACC1 EXPONENT.
         subb   #0x7F        ; TAKE OUT BIAS.
         negb                ;MAKE IT NEGATIVE.
         addb   #23          ;ADD IN THE NUMBER OF MANTISSA BITS -1.
         bra    FLTASC17     ;GO CHECK TO SEE IF WE NEED TO SHIFT AT ALL.
FLTASC7:  lsr    __fpacc1_mantissa    
						; SHIFT MANTISSA TO THE RIGHT BY THE RESULT (MAKE
         ror    __fpacc1_mantissa+1   ;THE NUMBER AN INTEGER).
         ror    __fpacc1_mantissa+2
         decb                ;DONE SHIFTING?
FLTASC17: bne    FLTASC7     ; NO. KEEP GOING.
         ldaa   #1           ;GET INITIAL VALUE OF "DIGITS AFTER D.P." COUNT.
         staa   3,y          ;INITIALIZE IT.
         ldaa   2,y          ;GET DECIMAL EXPONENT.
         adda   #8           ;ADD THE NUMBER OF DECIMAL +1 TO THE EXPONENT.
;                            ;WAS THE ORIGINAL NUMBER > 9999999?
         bmi    FLTASC8      ;YES. MUST BE REPRESENTED IN SCIENTIFIC NOTATION.
         cmpa   #8           ;WAS THE ORIGINAL NUMBER < 1?
         bhs    FLTASC8      ;YES. MUST BE REPRESENTED IN SCIENTIFIC NOTATION.
         deca                ;NO. NUMBER CAN BE REPRESENTED IN 7 DIGITS.
         staa   3,y          ;MAKE THE DECIMAL EXPONENT THE DIGIT COUNT BEFORE
;                            ;THE DECIMAL POINT.
         ldaa   #2           ;SETUP TO ZERO THE DECIMAL EXPONENT.
FLTASC8:  suba   #2          ; SUBTRACT 2 FROM THE DECIMAL EXPONENT.
         staa   2,y          ;SAVE THE DECIMAL EXPONENT.
         tst    3,y          ;DOES THE NUMBER HAVE AN INTEGER PART? (EXP. >0)
         bgt    FLTASC9      ;YES. GO PUT IT OUT.9
         ldaa   #'.         ;NO. GET DECIMAL POINT.
         ldx    0,y          ;GET POINTER TO BUFFER.
         staa   0,x          ;PUT THE DECIMAL POINT IN THE BUFFER.
         inx                 ;POINT TO NEXT BUFFER LOCATION.
         tst    3,y          ;IS THE DIGIT COUNT TILL EXPONENT =0?
         beq    FLTASC18     ;NO. NUMBER IS <.1
         ldaa   #'0         ;YES. FORMAT NUMBER AS .0XXXXXXX
         staa   0,x          ;PUT THE 0 IN THE BUFFER.
         inx                 ;POINT TO THE NEXT LOCATION.
FLTASC18: stx    0,y         ; SAVE NEW POINTER VALUE.
FLTASC9:  ldx    #DECDIG     ; POINT TO THE TABLE OF DECIMAL DIGITS.
         ldaa   #7           ;INITIALIZE THE THE NUMBER OF DIGITS COUNT.
         staa   5,y
FLTASC10: clr    4,y         ; CLEAR THE DECIMAL DIGIT ACCUMULATOR.
FLTASC11: ldd    __fpacc1_mantissa+1  ; GET LOWER 16 BITS OF MANTISSA.
         subd   1,x          ;SUBTRACT LOWER 16 BITS OF CONSTANT.
         std    __fpacc1_mantissa+1   ;SAVE RESULT.
         ldaa   __fpacc1_mantissa     ;GET UPPER 8 BITS.
         sbca   0,x          ;SUBTRACT UPPER 8 BITS.
         staa   __fpacc1_mantissa     ;SAVE RESULT. UNDERFLOW?
         bcs    FLTASC12     ;YES. GO ADD DECIMAL NUMBER BACK IN.
         inc    4,y          ;ADD 1 TO DECIMAL NUMBER.
         bra    FLTASC11     ;TRY ANOTHER SUBTRACTION.
FLTASC12: ldd    __fpacc1_mantissa+1  ; GET FPACC1 MANTISSA LOW 16 BITS.
         addd   1,x          ;ADD LOW 16 BITS BACK IN.
         std    __fpacc1_mantissa+1   ;SAVE THE RESULT.
         ldaa   __fpacc1_mantissa     ;GET HIGH 8 BITS.
         adca   0,x          ;ADD IN HIGH 8 BITS OF CONSTANT.
         staa   __fpacc1_mantissa     ;SAVE RESULT.
         ldaa   4,y          ;GET DIGIT.
         adda   #0x30        ; MAKE IT ASCII.
         pshx                ;SAVE POINTER TO CONSTANTS.
         ldx    0,y          ;GET POINTER TO BUFFER.
         staa   0,x          ;PUT DIGIT IN BUFFER.
         inx                 ;POINT TO NEXT BUFFER LOCATION.
         dec    3,y          ;SHOULD WE PUT A DECIMAL POINT IN THE BUFFER YET?
         bne    FLTASC16     ;NO. CONTINUE THE CONVERSION.
         ldaa   #'.         ;YES. GET DECIMAL POINT.
         staa   0,x          ;PUT IT IN THE BUFFER.
         inx                 ;POINT TO THE NEXT BUFFER LOCATION.
FLTASC16: stx    0,y         ; SAVE UPDATED POINTER.
         pulx                ;RESTORE POINTER TO CONSTANTS.
         inx                 ;POINT TO NEXT CONSTANT.
         inx
         inx
         dec    5,y          ;DONE YET?
         bne    FLTASC10     ;NO. CONTINUE CONVERSION OF "MANTISSA".
         ldx    0,y          ;YES. POINT TO BUFFER STRING BUFFER.
FLTASC13: dex                ; POINT TO LAST CHARACTER PUT IN THE BUFFER.
         ldaa   0,x          ;GET IT.
         cmpa   #0x30        ; WAS IT AN ASCII 0?
         beq    FLTASC13     ;YES. REMOVE TRAILING ZEROS.
         inx                 ;POINT TO NEXT AVAILABLE LOCATION IN BUFFER.
         ldab   2,y          ;DO WE NEED TO PUT OUT AN EXPONENT?
         beq    FLTASC15     ;NO. WE'RE DONE.
         ldaa   #'E         ;YES. PUT AN 'E' IN THE BUFFER.
         staa   0,x
         inx                 ;POINT TO NEXT BUFFER LOCATION.
         ldaa   #'+         ;ASSUME EXPONENT IS POSITIVE.
         staa   0,x          ;PUT PLUS SIGN IN THE BUFFER.
         tstb                ;IS IT REALLY MINUS?
         bpl    FLTASC14     ;NO. IS'S OK AS IS.
         negb                ;YES. MAKE IT POSITIVE.
         ldaa   #'-         ;PUT THE MINUS SIGN IN THE BUFFER.
         staa   0,x
FLTASC14: inx                ; POINT TO NEXT BUFFER LOCATION.
         stx    0,y          ;SAVE POINTER TO STRING BUFFER.
         clra                ;SET UP FOR DIVIDE.
         ldx    #10          ;DIVIDE DECIMAL EXPONENT BY 10.
         idiv
         pshb                ;SAVE REMAINDER.
         xgdx                ;PUT QUOTIENT IN D.
         addb   #0x30        ; MAKE IT ASCII.
         ldx    0,y          ;GET POINTER.
         stab   0,x          ;PUT NUMBER IN BUFFER.
         inx                 ;POINT TO NEXT LOCATION.
         pulb                ;GET SECOND DIGIT.
         addb   #0x30        ; MAKE IT ASCII.
         stab   0,x          ;PUT IT IN THE BUFFER.
         inx                 ;POINT TO NEXT LOCATION.
FLTASC15: clr    0,x         ; TERMINATE STRING WITH A ZERO BYTE.
         pulx                ;CLEAR LOCALS FROM STACK.
         pulx
         pulx
         jsr    __pull_fpacc2     ;RESTORE FPACC2.
         pula
         staa   __fpacc1_mantissa_sign
         pulx                ;RESTORE FPACC1.
         stx    __fpacc1_mantissa+1
         pulx
         stx    __fpacc1_exp
         pulx                ;POINT TO THE START OF THE ASCII STRING.
         rts                 ;RETURN.
;
;
DECDIG:   ;
         .byte    0x0F,0x42,0x40  ;DECIMAL 1,000,000
         .byte    0x01,0x86,0xA0  ;DECIMAL   100,000
         .byte    0x00,0x27,0x10  ;DECIMAL    10,000
         .byte    0x00,0x03,0xE8  ;DECIMAL     1,000
         .byte    0x00,0x00,0x64  ;DECIMAL       100
         .byte    0x00,0x00,0x0A  ;DECIMAL        10
         .byte    0x00,0x00,0x01  ;DECIMAL         1
;
;
P9999999: ;            ;CONSTANT 999999.9
         .byte    0x49,0x74,0x23,0xFE
;
N9999999: ;            ;CONSTANT 9999999.
         .byte    0x4B,0x18,0x96,0x7F
