		.area	text
__fpmod:: ;            FLOATING POINT MODULUS
         jsr    __fpdiv       	;DO DIVISION
         jsr    __push_fpacc2	;SAVE ARGUMENT
         jsr    __fp_intfrac	;FIND FRACTIONAL PART
         jsr    __pull_fpacc2	;RECOVER ARGUMENT
;                            ;FALL THROUGH TO MULTIPLY
;
;*****************************************************************************
;                                                                            *
;                      FPMULT: FLOATING POINT MULTIPLY                       *
;                                                                            *
;       THIS FLOATING POINT MULTIPLY ROUTINE MULTIPLIES "FPACC1" BY          *
;       "FPACC2" AND PLACES THE RESULT IN TO FPACC1. FPACC2 REMAINS          *
;       UNCHANGED.                                                           *
;                          WORST CASE = 480 CYCLES = 240 uS @ 2MHz           *
;                                                                            *
;*****************************************************************************
;
__fpmul::
         tst    __fpacc1_exp     ;CHECK TO SEE IF FPACC1 IS ZERO.
         beq    FPMULT3      ;IT IS. ANSWER IS 0.
         tst    __fpacc2_exp     ;CHECK TO SEE IF FPACC2 IS ZERO.
         bne    FPMULT8      ;IT IS NOT. GO DO MULTIPLY
FPMULT3: ldd    #0          ; ZERO RESULT
         staa   __fpacc1_mantissa_sign
         std    __fpacc1_exp
         std    __fpacc1_mantissa+1
         rts                 ;RETURN.
FPMULT8: ldaa   __fpacc1_mantissa_sign    ; GET FPACC1 EXPONENT.
         eora   __fpacc2_mantissa_sign     ;SET THE SIGN OF THE RESULT.
         staa   __fpacc1_mantissa_sign     ;SAVE THE SIGN OF THE RESULT.
         ldaa   __fpacc1_exp     ;GET FPACC1 EXPONENT.
         adda   __fpacc2_exp     ;ADD IT TO FPACC2 EXPONENT.
         bpl    FPMULT1      ;IF RESULT IS MINUS AND
         bcc    FPMULT2      ;THE CARRY IS SET THEN:
__fp_return_max::  ldaa   #OVFERR     ; OVERFLOW ERROR.
__fp_return_div0:: ldx    #0xFFFF     ;  MAXIMUM MAGNITUDE RESULT
         bra    FPMULT7      ;DO IT AND RETURN.
FPMULT1: bcs    FPMULT2     ; IF RESULT IS PLUS & THE CARRY IS SET THEN ALL OK.
__fp_return_zero:: 
		 ldaa   #UNFERR     ; ELSE UNDERFLOW ERROR OCCURED.
         ldx    #0           ;ZERO RESULT
         stx    __fpacc1_mantissa+2   ;THIS CLEARS __fpacc1_mantissa_sign BYTE
FPMULT7: stx    __fpacc1_exp
         stx    __fpacc1_mantissa+1
         sec                 ;FLAG ERROR.
         rts                 ;RETURN.
FPMULT2: adda   #0x82       ;  ADD BIAS BACK IN THAT WE LOST.
         staa   __fpacc1_exp     ;SAVE THE NEW EXPONENT.
         ldx    #0
         pshx                ;CREATE PARTIAL PRODUCT REGISTER AND COUNTER.
         pshx
         tsx                 ;POINT TO THE VARIABLES.
         jsr    UMULT        ;GO MULTIPLY THE "INTEGER" MANTISSAS.
         tst    0,x          ;DOES RESULT NEED TO BE NORMALIZED?
         bmi    FPMROUND     ;NO - GO ROUND RESULT
         rol    3,x          ;FIRST NORMALIZE RESULT
         rol    2,x
         rol    1,x
         rol    0,x
         dec    __fpacc1_exp     ;NOW DECREMENT EXPONENT
FPMROUND:tst    3,x         ; CHECK MSB OF BYTE 4 (TO BE DISCARDED LATER)
         bpl    FPMULT4      ;NO ROUNDING NECESSARY
         ldaa   2,x          ;ROUND LSB UP
         inca                ;INCREMENT RESULT LSB
         staa   2,x          ;PUT LSB BACK
         bne    FPMULT4      ;IF NO CARRY TO RESULT BYTES 1 AND 2
         ldab   #1           ;SET D REGISTER TO 1 (ACCA IS ALREADY 0)
         addd   0,x          ;INCREMENT BYTES 1 AND 2
         bcc    FPMULT5      ;NO OVERFLOW FROM BYTES 1 AND 2
         rora                ;RESULT CHANGES FROM 0x7FFFFF TO 0x800000
         inc    __fpacc1_exp     ;EXPONENT INCREMENTED (BACK TO WHERE IT WAS)
FPMULT5: std    0,x         ; PUT BYTES 1 AND 2 BACK
FPMULT4: pulx               ; RETRIEVE BYTES 1 AND 2
         stx    __fpacc1_mantissa     ;STORE IN MANTISSA HIGH BYTES
         pula                ;RETRIEVE BYTE 3 (LSB)
         staa   __fpacc1_mantissa+2   ;STORE IN MANTISSA LOW BYTE
         ins                 ;DISCARD BYTE 4
         tst    __fpacc1_exp     ;WAS THERE AN UNDERFLOW ERROR?
         beq    __fp_return_zero      ;YES. RETURN ERROR.
         clrb                ;SHOW NO ERRORS.
         rts
;
;
UMULT:   ;
         ldaa   __fpacc2_mantissa+2   ;GET MULTIPLIER LSB
         ldab   __fpacc1_mantissa+2   ;GET MULTIPLICAND LSB
         mul
         staa   1,x          ;TEMPORARILY SAVE RESULT MSB; DISCARD LSB (BYTE 6)
         ldaa   __fpacc2_mantissa+1   ;GET MULTIPLIER NSB
         ldab   __fpacc1_mantissa+2   ;GET MULTIPLICAND LSB
         mul
         addd   0,x          ;ADD IN LAST PARTIAL RESULT
         std    0,x          ;TEMPORARILY SAVE RESULT
         ldaa   __fpacc2_mantissa+2   ;GET MULTIPLIER LSB
         ldab   __fpacc1_mantissa+1   ;GET MULTIPLICAND NSB
         mul
         addd   0,x          ;ADD IN LAST PARTIAL RESULT
         staa   3,x          ;SAVE PARTIAL PRODUCT BYTE 4; DISCARD LSB (BYTE 5)
         bcc    UMULT1       ;IF NO CARRY OUT TO PRODUCT BYTE 3
         inc    2,x          ;CARRY TO PRODUCT BYTE 3
UMULT1:  clr    0,x         ; ZERO PRODUCT BYTES 1 & 2 (USED FOR TEMP. STORAGE)
         clr    1,x
         ldaa   __fpacc2_mantissa     ;GET MULTIPLIER MSB
         ldab   __fpacc1_mantissa+2   ;GET MULTIPLICAND LSB
         mul
         addd   2,x          ;ADD IN LAST PARTIAL RESULT
         std    2,x          ;SAVE IN PARTIAL PRODUCT BYTES 3 AND 4
__fp_remcomp:: 
		 ldaa   __fpacc2_mantissa+1  ; GET MULTIPLIER NSB
         ldab   __fpacc1_mantissa+1   ;GET MULTIPLICAND NSB
         mul
         addd   2,x          ;ADD IN LAST PARTIAL RESULT
         std    2,x          ;SAVE IN PARTIAL PRODUCT BYTES 3 AND 4
         bcc    UMULT2       ;IF NO CARRY OUT TO PRODUCT BYTE 2
         inc    1,x          ;CARRY TO PRODUCT BYTE 2
UMULT2:  ldaa   __fpacc2_mantissa+2  ; GET MULTIPLIER LSB
         ldab   __fpacc1_mantissa     ;GET MULTIPLICAND MSB
         mul
         addd   2,x          ;ADD IN LAST PARTIAL RESULT
         std    2,x          ;SAVE IN PARTIAL PRODUCT BYTES 3 AND 4
         bcc    UMULT3       ;IF NO CARRY OUT TO PRODUCT BYTE 2
         inc    1,x          ;CARRY TO PRODUCT BYTE 2
UMULT3:  ldaa   __fpacc2_mantissa    ; GET MULTIPLIER MSB
         ldab   __fpacc1_mantissa+1   ;GET MULTIPLICAND NSB
         mul
         addd   1,x          ;ADD IN LAST PARTIAL RESULT
         std    1,x          ;SAVE IN PARTIAL PRODUCT BYTES 2 AND 3
         bcc    UMULT4       ;IF NO CARRY OUT TO PRODUCT BYTE 1
         inc    0,x          ;CARRY TO PRODUCT BYTE 1
UMULT4:  ldaa   __fpacc2_mantissa+1  ; GET MULTIPLIER NSB
         ldab   __fpacc1_mantissa     ;GET MULTIPLICAND MSB
         mul
         addd   1,x          ;ADD IN LAST PARTIAL RESULT
         std    1,x          ;SAVE IN PARTIAL PRODUCT BYTES 2 AND 3
         bcc    UMULT5       ;IF NO CARRY OUT TO PRODUCT BYTE 1
         inc    0,x          ;CARRY TO PRODUCT BYTE 1
UMULT5:  ldaa   __fpacc2_mantissa    ; GET MULTIPLIER MSB
         ldab   __fpacc1_mantissa     ;GET MULTIPLICAND MSB
         mul
         addd   0,x          ;ADD IN LAST PARTIAL RESULT
         std    0,x          ;SAVE IN PARTIAL PRODUCT BYTES 2 AND 3
         rts                 ;RETURN TO DO NORMALIZATION AND ROUNDING
