;*****************************************************************************
;                                                                            *
;                         SQUARE ROOT SUBROUTINE                             *
;                                                                            *
;        This routine is used to calculate the square root of the floating   *
;        point number in FPACC1.  If the number in FPACC1 is negative an     *
;        error is returned.                                                  *
;                                                                            *
;                          WORSE CASE = 16354 CYCLES = 8177 uS @ 2MHz        *
;                                                                            *
;*****************************************************************************
;
;
	.area	text
_sqrt::
	pshb
	psha
	tsx
	ldab	#4
	abx
	jsr		__to_fpacc1
	bsr		FLTSQRT
	tsx
	pulx
	jsr		__from_fpacc1
	rts

FLTSQRT:
         ldx    #__fpacc1_exp    ;POINT TO FPACC1.
         jsr    __fp_is_acc1_zero        ;IS IT ZERO?
         bne    FLTSQR1      ;NO. CHECK FOR NEGATIVE.
         rts                 ;YES. RETURN.
FLTSQR1:  tst    __fpacc1_mantissa_sign    ; IS THE NUMBER NEGATIVE?
         bpl    FLTSQR2      ;NO. GO TAKE ITS SQUARE ROOT.
         ldaa   #NSQRTERR    ;YES. ERROR.
         sec                 ;FLAG ERROR.
         rts                 ;RETURN.
FLTSQR2:  jsr    __push_fpacc2    ; SAVE FPACC2.
         ldaa   #4           ;GET ITERATION LOOP COUNT.
         psha                ;SAVE IT ON THE STACK.
         ldx    __fpacc1_mantissa+1   ;SAVE INITIAL NUMBER.
         pshx
         ldx    __fpacc1_exp
         pshx
         tsy                 ;POINT TO IT.
         jsr    __fp_xfer_acc1_to_acc2      ;TRANSFER FPACC1 TO FPACC2.
         ldaa   __fpacc2_exp     ;GET FPACC1 EXPONENT.
         suba   #0x7E        ; REMOVE BIAS FROM EXPONENT.
         inca                ;COMPENSATE FOR ODD EXPONENTS (GIVES CLOSER GUESS)
         bpl    FLTSQR3      ;IF NUMBER >1 DIVIDE EXPONENT BY 2 & ADD BIAS.
         lsra                ;IF <1 JUST DIVIDE IT BY 2.
         bra    FLTSQR4      ;GO CALCULATE THE SQUARE ROOT.
FLTSQR3:  lsra               ; DIVIDE EXPONENT BY 2.
         adda   #0x7E        ; ADD BIAS BACK IN.
FLTSQR4:  staa   __fpacc2_exp    ; SAVE EXPONENT/2.
FLTSQR5: jsr    __fpdiv      ; DIVIDE THE ORIGINAL NUMBER BY THE GUESS.
         jsr    __fpadd       ;ADD THE "GUESS" TO THE QUOTIENT.
         dec    __fpacc1_exp     ;DIVIDE THE RESULT BY 2 TO PRODUCE A NEW GUESS.
         jsr    __fp_xfer_acc1_to_acc2      ;PUT THE NEW GUESS INTO FPACC2.
         ldd    0,y          ;GET THE ORIGINAL NUMBER.
         std    __fpacc1_exp     ;PUT IT BACK IN FPACC1.
         ldd    2,y          ;GET MANTISSA LOWER 16 BITS.
         std    __fpacc1_mantissa+1
         dec    4,y          ;BEEN THROUGH THE LOOP 4 TIMES?
         bne    FLTSQR5      ;NO. KEEP GOING.
         ldd    __fpacc2_exp     ;THE FINAL GUESS IS THE ANSWER.
         std    __fpacc1_exp     ;PUT IT IN FPACC1.
         ldd    __fpacc2_mantissa+1
         std    __fpacc1_mantissa+1
         pulx                ;GET RID OF ORIGINAL NUMBER.
         pulx
         ins                 ;GET RID OF LOOP COUNT VARIABLE.
         jsr    __pull_fpacc2     ;RESTORE FPACC2.
         clrb                ;NO ERRORS.
         rts
