;*****************************************************************************
;                                                                            *
;                        ASCII TO FLOATING POINT ROUTINE                     *
;                                                                            *
;       This routine will accept most any ASCII floating point format        *
;       and return a 32-bit floating point number.  The following are        *
;       some examples of legal ASCII floating point numbers.                 *
;                                                                            *
;       20.095                                                               *
;       0.125                                                                *
;       7.2984E10                                                            *
;       167.824E5                                                            *
;       5.9357E-7                                                            *
;       500                                                                  *
;                                                                            *
;       The floating point number returned is in "FPACC1".                   *
;                                                                            *
;                                                                            *
;       The exponent is biased by 127 to facilitate floating point           *
;       comparisons.  A pointer to the ASCII string is passed to the         *
;       routine in the X-register.                                           *
;                                                                            *
;                                                                            *
;*****************************************************************************
;
;
;
;        LOCAL VARIABLES (ON STACK POINTED TO BY Y)
;
EXPSIGN   =    0           ; EXPONENT SIGN (0=+, FF=-).
PWR10EXP  =    1           ; POWER 10 EXPONENT.
;
;
	.area	text
;

_atof::
	pshb
	psha
	tsx
	ldx		4,x			; move buffer to x
	bsr		ASCFLT		; convert
	tsx
	pulx
	jsr		__from_fpacc1	; return val
	rts
	; arg: 		reg X
	; return:	FPACC1
ASCFLT:
         pshx               ; SAVE POINTER TO ASCII STRING.
         jsr    __push_fpacc2    ; SAVE FPACC2.
         ldx    #0          ; PUSH ZEROS ON STACK TO INITIALIZE LOCALS.
         pshx               ; ALLOCATE 2 BYTES FOR LOCALS.
         stx    __fpacc1_exp    ; CLEAR FPACC1.
         stx    __fpacc1_exp+2
         clr    __fpacc1_mantissa_sign    ; MAKE THE MANTISSA SIGN POSITIVE INITIALLY.
         tsy                ; POINT TO LOCALS.
         ldx    6,y         ; GET POINTER TO ASCII STRING.
ASCFLT1:  ldaa   0,x        ;  GET 1ST CHARACTER IN STRING.
         jsr    NUMERIC     ; IS IT A NUMBER.
         bcs    ASCFLT4     ; YES. GO PROCESS IT.
;
;        LEADING MINUS SIGN ENCOUNTERED?
;
ASCFLT2:  cmpa  #'-       ;  NO. IS IT A MINUS SIGN?
         bne    ASCFLT3     ; NO. GO CHECK FOR DECIMAL POINT.
         com    __fpacc1_mantissa_sign    ; YES. SET MANTISSA SIGN. LEADING MINUS BEFORE?
         inx                ; POINT TO NEXT CHARACTER.
         ldaa   0,x         ; GET IT.
         jsr    NUMERIC     ; IS IT A NUMBER?
         bcs    ASCFLT4     ; YES. GO PROCESS IT.
;
;        LEADING DECIMAL POINT?
;

ASCFLT3:  cmpa   #'.        ;  IS IT A DECIMAL POINT?
         bne    ASCFLT5     ; NO. FORMAT ERROR.
         inx                ; YES. POINT TO NEXT CHARACTER.
         ldaa   0,x         ; GET IT.
         jsr    NUMERIC     ; MUST HAVE AT LEAST ONE DIGIT AFTER D.P.
         bcc    ASCFLT5     ; GO REPORT ERROR.
         jmp    ASCFLT11    ; GO BUILD FRACTION.
;
;        FLOATING POINT FORMAT ERROR
;
ASCFLT5:  ins               ;  DE-ALLOCATE LOCALS.
         ins
         jsr    __pull_fpacc2    ; RESTORE FPACC2.
         pulx               ; GET POINTER TO TERMINATING CHARACTER IN STRING.
         ldaa   #FLTFMTER   ; FORMAT ERROR.
         sec                ; SET ERROR FLAG.
         rts                ; RETURN.
;
;        PRE DECIMAL POINT MANTISSA BUILD
;
ASCFLT4:  ldaa   0,x
         jsr    NUMERIC
         bcc    ASCFLT10
         jsr    ADDNXTD
         inx
         bcc    ASCFLT4
;
;        PRE DECIMAL POINT MANTISSA OVERFLOW
;
ASCFLT6:  inc    __fpacc1_exp    ; INC FOR EACH DIGIT ENCOUNTERED PRIOR TO D.P.
         ldaa   0,x          ;GET NEXT CHARACTER.
         inx                 ;POINT TO NEXT.
         jsr    NUMERIC      ;IS IT S DIGIT?
         bcs    ASCFLT6      ;YES. KEEP BUILDING POWER 10 MANTISSA.
         cmpa   #'.         ;NO. IS IT A DECIMAL POINT?
         bne    ASCFLT7      ;NO. GO CHECK FOR THE EXPONENT.
;
;        ANY FRACTIONAL DIGITS ARE NOT SIGNIFIGANT
;
ASCFLT8:  ldaa   0,x         ; GET THE NEXT CHARACTER.
         jsr    NUMERIC      ;IS IT A DIGIT?
         bcc    ASCFLT7      ;NO. GO CHECK FOR AN EXPONENT.
         inx                 ;POINT TO THE NEXT CHARACTER.
         bra    ASCFLT8      ;FLUSH REMAINING DIGITS.
ASCFLT7:  cmpa   #'E        ; NO. IS IT THE EXPONENT?
         beq    ASCFLT13     ;YES. GO PROCESS IT.
         cmpa   #'e         ;IS IT THE exponent?
         beq    ASCFLT13     ;YES. GO PROCESS IT.
         jmp    FINISH       ;NO. GO FINISH THE CONVERSION.
;
;        PROCESS THE EXPONENT
;
ASCFLT13: inx                 ;POINT TO NEXT CHARACTER.
         ldaa   0,x         ; GET THE NEXT CHARACTER.
         jsr    NUMERIC     ; SEE IF IT'S A DIGIT.
         bcs    ASCFLT9     ; YES. GET THE EXPONENT.
         cmpa   #'-        ; NO. IS IT A MINUS SIGN?
         beq    ASCFLT15    ; YES. GO FLAG A NEGATIVE EXPONENT.
         cmpa   #'+        ; NO. IS IT A PLUS SIGN?
         beq    ASCFLT16    ; YES. JUST IGNORE IT.
         bra    ASCFLT5     ; NO. FORMAT ERROR.
ASCFLT15: com    EXPSIGN,y  ;  FLAG A NEGATIVE EXPONENT. IS IT 1ST?
ASCFLT16: inx               ;  POINT TO NEXT CHARACTER.
         ldaa   0,x         ; GET NEXT CHARACTER.
         jsr    NUMERIC     ; IS IT A NUMBER?
         bcc    ASCFLT5     ; NO. FORMAT ERROR.
ASCFLT9:  suba   #0x30      ;   MAKE IT BINARY.
         staa   PWR10EXP,y  ; BUILD THE POWER 10 EXPONENT.
         inx                ; POINT TO NEXT CHARACTER.
         ldaa   0,x         ; GET IT.
         jsr    NUMERIC     ; IS IT NUMERIC?
         bcc    ASCFLT14    ; NO. GO FINISH UP THE CONVERSION.
         ldab   PWR10EXP,y  ; YES. GET PREVIOUS DIGIT.
         lslb               ; MULT. BY 2.
         lslb               ; NOW BY 4.
         addb   PWR10EXP,y  ; BY 5.
         lslb               ; BY 10.
         suba   #0x30       ;  MAKE SECOND DIGIT BINARY.
         aba                ; ADD IT TO FIRST DIGIT.
         staa   PWR10EXP,y
         inx                ; POINT TO CHARACTER FOLLOWING EXPONENT
         cmpa   #38         ; IS THE EXPONENT OUT OF RANGE?
         bhi    ASCFLT5     ; YES. REPORT ERROR.
ASCFLT14: ldaa   PWR10EXP,y ;  GET POWER 10 EXPONENT.
         tst    EXPSIGN,y   ; WAS IT NEGATIVE?
         bpl    ASCFLT12    ; NO. GO ADD IT TO BUILT 10 PWR EXPONENT.
         nega
ASCFLT12: adda   __fpacc1_exp   ;  FINAL TOTAL PWR 10 EXPONENT.
         staa   __fpacc1_exp    ; SAVE RESULT.
         bra    FINISH      ; GO FINISH UP CONVERSION.
;
;        PRE-DECIMAL POINT NON-DIGIT FOUND, IS IT A DECIMAL POINT?
;
ASCFLT10: cmpa   #'.       ;  IS IT A DECIMAL POINT?
         bne    ASCFLT7     ; NO. GO CHECK FOR THE EXPONENT.
         inx                ; YES. POINT TO NEXT CHARACTER.
;
;        POST DECIMAL POINT PROCESSING
;
ASCFLT11: ldaa   0,x        ;  GET NEXT CHARACTER.
         jsr    NUMERIC     ; IS IT NUMERIC?
         bcc    ASCFLT7     ; NO. GO CHECK FOR EXPONENT.
         bsr    ADDNXTD     ; YES. ADD IN THE DIGIT.
         inx                ; POINT TO THE NEXT CHARACTER.
         bcs    ASCFLT8     ; IF OVER FLOW, FLUSH REMAINING DIGITS.
         dec    __fpacc1_exp    ; ADJUST THE 10 POWER EXPONENT.
         bra    ASCFLT11    ; PROCESS ALL FRACTIONAL DIGITS.
;
;
;
ADDNXTD:  ldaa   __fpacc1_mantissa    ; GET UPPER 8 BITS.
         staa   __fpacc2_mantissa     ;COPY INTO FPAC2.
         ldd    __fpacc1_mantissa+1   ;GET LOWER 16 BITS OF MANTISSA.
         std    __fpacc2_mantissa+1   ;COPY INTO FPACC2.
         lsld                ;MULT. BY 2.
         rol    __fpacc1_mantissa     ;OVERFLOW?
         bcs    ADDNXTD1     ;YES. DON'T ADD THE DIGIT IN.
         lsld                ;MULT BY 4.
         rol    __fpacc1_mantissa     ;OVERFLOW?
         bcs    ADDNXTD1     ;YES. DON'T ADD THE DIGIT IN.
         addd   __fpacc2_mantissa+1   ;BY 5.
         psha                ;SAVE A.
         ldaa   __fpacc1_mantissa     ;GET UPPER 8 BITS.
         adca   __fpacc2_mantissa     ;ADD IN UPPER 8 BITS, CARRY FROM LOWER 16 BITS.
         staa   __fpacc1_mantissa     ;SAVE IT.
         pula                ;RESTORE A.
         bcs    ADDNXTD1     ;OVERFLOW? IF SO DON'T ADD IT IN.
         lsld                ;BY 10.
         rol    __fpacc1_mantissa
         std    __fpacc1_mantissa+1   ;SAVE THE LOWER 16 BITS.
         bcs    ADDNXTD1     ;OVERFLOW? IF SO DON'T ADD IT IN.
         ldab   0,x          ;GET CURRENT DIGIT.
         subb   #0x30        ; MAKE IT BINARY.
         clra                ;16-BIT.
         addd   __fpacc1_mantissa+1   ;ADD IT IN TO TOTAL.
         std    __fpacc1_mantissa+1   ;SAVE THE RESULT.
         ldaa   __fpacc1_mantissa     ;GET UPPER 8 BITS.
         adca   #0           ;ADD IN POSSIBLE CARRY. OVERFLOW?
         bcs    ADDNXTD1     ;YES. COPY OLD MANTISSA FROM FPACC2.
         staa   __fpacc1_mantissa     ;NO. EVERYHING OK.
         rts                 ;RETURN.
ADDNXTD1: ldd    __fpacc2_mantissa+1  ; RESTORE THE ORIGINAL MANTISSA BECAUSE
         std    __fpacc1_mantissa+1   ;OF OVERFLOW.
         ldaa   __fpacc2_mantissa
         staa   __fpacc1_mantissa
         rts                 ;RETURN.
;
;
;
;        NOW FINISH UP CONVERSION BY MULTIPLYING THE RESULTANT MANTISSA
;        BY 10 FOR EACH POSITIVE POWER OF 10 EXPONENT RECIEVED OR BY .1
;        (DIVIDE BY 10) FOR EACH NEGATIVE POWER OF 10 EXPONENT RECIEVED.
;
;
FINISH:   
         stx    6,y          ;SAVE POINTER TO TERMINATING CHARACTER IN STRING.
         ldx    #__fpacc1_exp    ;POINT TO FPACC1.
         jsr    __fp_is_acc1_zero        ;SEE IF THE NUMBER IS ZERO.
         beq    FINISH3      ;QUIT IF IT IS.
         ldaa   __fpacc1_exp     ;GET THE POWER 10 EXPONENT.
         staa   PWR10EXP,y   ;SAVE IT.
         ldaa   #0x7E+24     ; SET UP INITIAL EXPONENT (# OF BITS + BIAS).
         staa   __fpacc1_exp
         jsr    FPNORM       ;GO NORMALIZE THE MANTISSA.
         tst    PWR10EXP,y   ;IS THE POWER 10 EXPONENT POSITIVE OR ZERO?
         beq    FINISH3      ;IT'S ZERO, WE'RE DONE.
         bpl    FINISH1      ;IT'S POSITIVE MULTIPLY BY 10.
         ldx    #CONSTP1     ;NO. GET CONSTANT .1 (DIVIDE BY 10).
         jsr    __to_fpacc2     ;GET CONSTANT INTO FPACC2.
         neg    PWR10EXP,y   ;MAKE THE POWER 10 EXPONENT POSITIVE.
         bra    FINISH2      ;GO DO THE MULTIPLIES.
FINISH1:  ldx    #CONST10    ; GET CONSTANT '10' TO MULTIPLY BY.
         jsr    __to_fpacc2     ;GET CONSTANT INTO FPACC2.
FINISH2: jsr    __fpmul      ; GO MULTIPLY FPACC1 BY FPACC2, RESULT IN FPACC1.
         dec    PWR10EXP,y   ;DECREMENT THE POWER 10 EXPONENT.
         bne    FINISH2      ;GO CHECK TO SEE IF WE'RE DONE.
FINISH3:  ins                ; DE-ALLOCATE LOCALS.
         ins
         jsr    __pull_fpacc2     ;RESTORE FPACC2.
         pulx                ;GET POINTER TO TERMINATING CHARACTER IN STRING.
         rts                 ;RETURN WITH NUMBER IN FPACC1.
;
;
NUMERIC:  
         cmpa   #'0         ;IS IT LESS THAN AN ASCII 0?
         blo    NUMERIC1     ;YES. NOT NUMERIC.
         cmpa   #'9         ;IS IT GREATER THAN AN ASCII 9?
         bhi    NUMERIC1     ;YES. NOT NUMERIC.
         sec                 ;IT WAS NUMERIC. SET THE CARRY.
         rts                 ;RETURN.
NUMERIC1: clc                ; NON-NUMERIC CHARACTER. CLEAR THE CARRY.
         rts                 ;RETURN.
;
FPNORM:   
         ldx    #__fpacc1_exp    ;POINT TO FPACC1.
         jsr    __fp_is_acc1_zero        ;CHECK TO SEE IF IT'S 0.
         beq    FPNORM3      ;YES. JUST RETURN.
         tst    __fpacc1_mantissa     ;IS THE NUMBER ALREADY NORMALIZED?
         bmi    FPNORM3      ;YES. JUST RETURN..
FPNORM1:  ldd    __fpacc1_mantissa+1  ; GET THE LOWER 16 BITS OF THE MANTISSA.
FPNORM2:  dec    __fpacc1_exp    ; DECREMENT THE EXPONENT FOR EACH SHIFT.
         beq    FPNORM4      ;EXPONENT WENT TO 0. UNDERFLOW.
         lsld                ;SHIFT THE LOWER 16 BITS.
         rol    __fpacc1_mantissa     ;ROTATE THE UPPER 8 BITS. NUMBER NORMALIZED?
         bpl    FPNORM2      ;NO. KEEP SHIFTING TO THE LEFT.
         std    __fpacc1_mantissa+1   ;PUT THE LOWER 16 BITS BACK INTO FPACC1.
FPNORM3:  clc                ; SHOW NO ERRORS.
         rts                 ;YES. RETURN.
FPNORM4:  sec                ; FLAG ERROR.
         rts                 ;RETURN.
