;*****************************************************************************
;                                                                            *
;                     FLOATING POINT SUBTRACT SUBROUTINE                     *
;                                                                            *
;       This subroutine performs floating point subtraction ( FPACC1-FPACC2) *
;       by inverting the sign of FPACC2 and then calling __fpadd since        *
;       __fpadd performs complete signed addition.  Upon returning from       *
;       __fpadd the sign of FPACC2 is again inverted to leave it unchanged    *
;       from its original value.                                             *
;                                                                            *
;                          WORSE CASE = 601 CYCLES = 301 uS @ 2MHz           *
;                                                                            *
;*****************************************************************************
;
;
	.area	text
__fpsub::
         bsr   FLTSUB1		  ;INVERT SIGN.
         jsr    __fpadd       ;GO DO FLOATING POINT ADD.
FLTSUB1:
		 ldaa   __fpacc2_mantissa_sign    ; GET FPACC2 MANTISSA SIGN.
         eora   #0xFF        ; INVERT THE SIGN.
         staa   __fpacc2_mantissa_sign     ;PUT BACK.
         rts                 ;RETURN.
;
;*****************************************************************************
;                                                                            *
;     SEPARATE A FLOATING POINT NUMBER INTO INTEGER AND FRACTIONAL PARTS     *
;                                                                            *
;        This subroutine separates the floating point number in FPACC1 into  *
;        integer and fractional parts.  The fractional part is returned in   *
;        FPACC1, and the integer part is returned in FPACC2.                 *
;                                                                            *
;*****************************************************************************
;
__fp_intfrac::
         jsr    __fp_xfer_acc1_to_acc2              ;SAVE ARGUMENT IN FPACC2
         ldx    #__fpacc2_mantissa+2          ;POINT TO MANTISSA LS BYTE
         ldaa   __fpacc2_exp             ;GET EXPONENT
         suba   #0x97                ; REMOVE BIAS + 2^24
         bpl    __fpsub               ;ARGUMENT IS LARGE INTEGER
         ldab   #0x03                ; DO THIS 3 TIMES, MAXIMUM
INTFRAC1:adda   #0x08               ;  INCREASE EXPONENT
         bpl    INTFRAC2             ;IF IN RANGE
         clr    0,x                  ;CLEAR BYTE
         dex                         ;POINT TO NEXT MOST SIGNIFICANT BYTE
         decb                        ;DECREMENT COUNTER
         bne    INTFRAC1             ;IF COUNT NOT ZERO
         clr    0,x                  ;CLEAR EXPONENT FOR NUMBER LESS THAN 1
         rts                         ;NO SUBTRACTION NECESSARY FOR ZERO
INTFRAC2: ldab   #0x80               ;  SET UP MASK
INTFRAC3: deca                       ; DECREMENT COUNTER
         bmi    INTFRAC4             ;COUNTER < 0
         asrb                        ;SHIFT NEXT BIT IN MASK
         bra    INTFRAC3             ;KEEP GOING TILL COUNTER RUNS OUT
INTFRAC4: andb   0,x                 ; MASK MANTISSA BYTE
         stab   0,x                  ;SAVE MANTISSA BYTE
		 jmp	__fpsub
;                                    ;FALL THROUGH TO SUBTRACT FOR FRACTION PART
