;*****************************************************************************
;                                                                            *
;                        FLOATING POINT ARC SINE                             *
;                        FLOATING POINT ARC COSINE                           *
;                        FLOATING POINT ARC TANGENT                          *
;                                                                            *
;*****************************************************************************
;
	.area	text
;
FLTASIN:  ;
         ldaa   #ASINERR    ; ARC SINE NOT IMPLEMENTED
         sec
         rts
FLTACOS:  ;
         ldaa   #ACOSERR     ;ARC COSINE NOT IMPLEMENTED
         sec
         rts
_atan::
	pshb
	psha
	tsx
	ldab	#4
	abx
	jsr		__to_fpacc1
	bsr		FLTATAN
	tsx
	pulx
	jsr		__from_fpacc1
	rts
FLTATAN:  ;
         jsr    __push_fpacc2     ;SAVE FPACC2
         ldaa   __fpacc1_mantissa_sign     ;GET SIGN OF ARGUMENT
         psha                ;SAVE IT FOR RESULT
         clr    __fpacc1_mantissa_sign     ;WORK WITH POSITIVE NUMBER (FOR COMPARE)
         ldx    #ONE         ;POINT TO FLOATING POINT CONSTANT 1.0
         jsr    __to_fpacc2     ;PUT IT ON STACK
         jsr    __fpcmp       ;CHECK FOR ARGUMENT GREATER THAN 1.0
         ble    FLTATAN1     ;IF <= 1.0
         jsr    FLTRECIP     ;TAKE RECIPROCAL OF ARGUMENT
         ldaa   #0xFF        ; INDICATE > 1.0
         bra    FLTATAN2     ;CONTINUE
FLTATAN1: clra               ; INDICATE <= 1.0
FLTATAN2: psha               ; SAVE ON STACK
         pshx                ;CREATE STACK SPACE FOR ARGUMENT
         pshx
         tsx                 ;POINT TO STORAGE SPACE
         jsr    __from_fpacc1     ;PUT ARGUMENT ON STACK
         jsr    __fp_xfer_acc1_to_acc2      ;PUT POWER SERIES VARIABLE INTO FPACC2
         jsr    __fpmul       ;SQUARE IT
         ldx    #ATANTBL     ;POINT TO ARC TANGENT TABLE
         jsr    POLYNOM      ;COMPUTE ARC TANGENT
         tsx                 ;GET POINTER TO ARGUMENT
         jsr    __to_fpacc2     ;PUT IN FPACC2
         jsr    __fpmul       ;MULTIPLY TO GET RESULT
         pulx                ;DISCARD ARGUMENT
         pulx
         pula                ;FIND OUT IF ARGUMENT WAS > 1.0
         tsta
         beq    FLTATAN3     ;NO - LEAVE RESULT ALONE
         ldx    #PIOV2       ;YES - SUBTRACT IT FROM PI/2
         jsr    __to_fpacc2     ;PUT PI/2 ON STACK
         com    __fpacc1_mantissa_sign     ;MAKE RESULT NEGATIVE
         jsr    __fpadd       ;DO SUBTRACTION
FLTATAN3: jsr    RAD2DEG     ; CONVERT TO DEGREES
         pula                ;RETRIEVE ORIGINAL SIGN
         staa   __fpacc1_mantissa_sign     ;PUT IT IN RESULT
         jsr    __pull_fpacc2     ;RETRIEVE FPACC2
         clrb                ;NO ERRORS
         rts                 ;RETURN
;
PIOV2:    .byte    0x3F,0xC9,0x0F,0xDB  ;    1.5707963
;
ATANTBL:  ;
         .byte    0xBC,0xE2,0xDD,0x1B    ;     -1/19 + FUDGE FACTOR
         .byte    0x3D,0x70,0xF0,0xF1    ;     1/17
         .byte    0xBD,0x88,0x88,0x89    ;     -1/15
         .byte    0x3D,0x9D,0x89,0xD9    ;     1/13
         .byte    0xBD,0xBA,0x2E,0x8C    ;     -1/11
         .byte    0x3D,0xE3,0x8E,0x39    ;     1/9
         .byte    0xBE,0x12,0x49,0x25    ;     -1/7
         .byte    0x3E,0x4C,0xCC,0xCD    ;     1/5
         .byte    0xBE,0xAA,0xAA,0xAB    ;     -1/3
         .byte    0x3F,0x80,0x00,0x00    ;     1/1
         .byte    0xFF                   ;  END OF TABLE
;
;
;
;*****************************************************************************
;                                                                            *
;                        FLOATING POINT SINE & COSINE                        *
;                                                                            *
;*****************************************************************************
;
;
_sin::
	pshb
	psha
	tsx
	ldab	#4
	abx
	jsr		__to_fpacc1
	bsr		FLTSIN
	tsx
	pulx
	jsr		__from_fpacc1
	rts
FLTSIN:   ;
         jsr    __push_fpacc2    ; SAVE FLOATING ACCUMULATOR
         clra               ; OPERATION IS SINE, RESULT IS POSITIVE
         bra    SIN0        ; CONTINUE
;
_cos::
	pshb
	psha
	tsx
	ldab	#4
	abx
	jsr		__to_fpacc1
	bsr		FLTCOS
	tsx
	pulx
	jsr		__from_fpacc1
	rts
FLTCOS:   ;
         jsr    __push_fpacc2    ; SAVE FLOATING ACCUMULATOR
         ldaa   #0xF0       ;  OPERATION IS COSINE, RESULT IS POSITIVE
SIN0:     psha              ;  SAVE OPERATION/SIGN FLAG
         ldx    #N360       ; POINT TO FLOATING POINT CONSTANT 360.0
         jsr    __to_fpacc2    ; PUT IT INTO FP ACC 2
         bra    SIN1        ; CHECK FOR NEGATIVE ARGUMENT
SIN2:     jsr    __fpadd     ;  ADD 360 AND TRY AGAIN
SIN1:     ldaa   __fpacc1_mantissa_sign   ;  IS ARGUMENT NEGATIVE?
         bne    SIN2        ; YES - MUST BE MADE POSITIVE
         bra    SIN3        ; CHECK FOR ARGUMENT > 360.0
SIN4:     jsr    __fpsub     ;  SUBTRACT 360 AND TRY AGAIN
SIN3:     jsr    __fpcmp     ;  IS ARGUMENT > 360?
         bgt    SIN4        ; YES - MUST BE LESS THAN 360
         bsr    ANGRED      ; IF 180 < ARG < 360, ARG = 360 - ARG ...
         bcc    SIN5        ; ... AND CHANGE SIGN IF SIN FUNCTION.
         tsx                ; GET BACK OPERATION/SIGN FLAG
         ldaa   0,x         ; ARE WE DOING SINE?
         bmi    SIN5        ; NO - GO DO NEXT REDUCTION
         eora   #0x0F       ;  YES - SINE IS NEGATIVE IN QUADS 3 AND 4
         staa   0,x         ; PUT OPERATION/SIGN FLAG BACK
SIN5:     bsr    ANGRED     ;  IF 90 < ARG < 180, ARG = 180 - ARG ...
         bcc    SIN6        ; ... AND CHANGE SIGN IF COS FUNCTION.
         tsx                ; GET BACK OPERATION/SIGN FLAG
         ldaa   0,x         ; ARE WE DOING COSINE?
         bpl    SIN6        ; NO - GO DO NEXT REDUCTION
         eora   #0x0F       ;  YES - COSINE IS NEGATIVE IN QUADS 2 AND 3
         staa   0,x         ; PUT OPERATION/SIGN FLAG BACK
SIN6:     bsr    ANGRED     ;  IF 45 < ARG < 90, ARG = 90 - ARG ...
         bcc    SIN7        ; ... AND CHANGE OPERATION SIN <=> COS.
         tsx                ; GET BACK OPERATION/SIGN FLAG
         ldaa   0,x
         eora   #0xF0       ;  CHANGE SINE TO COSINE; COSINE TO SINE
         staa   0,x         ; PUT OPERATION/SIGN FLAG BACK
SIN7:     ldx    #NPID180   ;  POINT TO FLOATING POINT CONSTANT PI/180
         jsr    __to_fpacc2    ; LOAD INTO FLOATING ACCUMULATOR 2
         jsr    __fpmul      ; DO DEGREES TO RADIANS CONVERSION
         jsr    __fp_xfer_acc1_to_acc2     ; COPY ARGUMENT INTO FPACC2
         jsr    __fpmul      ; COMPUTE ARGUMENT^2
         pshx               ; CREATE STORAGE SPACE FOR ARGUMENT
         pshx
         tsx                ; GET POINTER TO STORAGE
         jsr    __from_fpacc2    ; SAVE ARGUMENT
         ldaa   4,x         ; ARE WE DOING SINE?
         bpl    SIN8        ; YES - LOAD SINE TABLE POINTER
         ldx    #COSTBL     ; NO - LOAD COSINE TABLE POINTER
         bra    SIN85
SIN8:     ldx    #SINTBL    ;  LOAD SINE TABLE POINTER
SIN85:    jsr    POLYNOM    ;  GO DO TAYLOR EXPANSION
         tsx
         ldaa   4,x         ; GET BACK OPERATION/SIGN FLAG
         asra               ; CHECK FOR NEGATIVE
         bcc    SIN9        ; IF POSITIVE, LEAVE RESULT ALONE
         com    __fpacc1_mantissa_sign    ; IF NEGATIVE, COMPLEMENT SIGN
SIN9:     tsta              ;  CHECK SINE/COSINE FLAG
         bmi    SIN10       ; IF COSINE, WE ARE FINISHED
         jsr    __to_fpacc2    ; GET ARGUMENT BACK
         jsr    __fpmul      ; FINAL COMPUTATION FOR SINE
SIN10:    pulx              ;  DISCARD STACK TEMPORARIES
         pulx
         ins
         jsr    __pull_fpacc2    ; RECOVER FLOATING ACCUMULATOR
         clrb               ; NO ERRORS
         rts                ; DONE
;
;
ANGRED:   ;
         dec    __fpacc2_exp    ; MAKE N/2 FOR COMPARE
         jsr    __fpcmp      ; IS ACC1 > N/2?
         bgt    ANGRED1     ; YES - REDUCE IT
         clc                ; NO REDUCTION
         rts                ; RETURN
ANGRED1:  inc    __fpacc2_exp   ;  RECOVER N
         com    __fpacc1_mantissa_sign    ; MAKE ACC1 NEGATIVE
         jsr    __fpadd      ; ACC1 = -ACC1 + N
         dec    __fpacc2_exp    ; BACK TO N/2
         sec                ; SIGNAL REDUCTION
         rts                ; RETURN
;
;
;
;
FLTINT:   ;
         jsr    __fp_intfrac     ; DO SEPARATION INTO INTEGER AND FRACTIONAL PARTS
;                           ; FALL THROUGH TO EXCHANGE INTEGER PART INTO FPACC1
EXG1AND2: ;
         ldd    __fpacc1_exp
         ldx    __fpacc2_exp
         std    __fpacc2_exp
         stx    __fpacc1_exp
         ldd    __fpacc1_mantissa+1
         ldx    __fpacc2_mantissa+1
         std    __fpacc2_mantissa+1
         stx    __fpacc1_mantissa+1
         ldaa   __fpacc1_mantissa_sign
         ldab   __fpacc2_mantissa_sign
         staa   __fpacc2_mantissa_sign
         stab   __fpacc1_mantissa_sign
         clrb
         rts                ; RETURN.
;
;
SINTBL:   ;
         .byte    0x36,0x38,0xEF,0x1D  ;       +(1/9!)
         .byte    0xB9,0x50,0x0D,0x01  ;       -(1/7!)
         .byte    0x3C,0x08,0x88,0x89  ;       +(1/5!)
         .byte    0xBE,0x2A,0xAA,0xAB  ;       -(1/3!)
ONE:     .byte    0x3F,0x80,0x00,0x00 ;        +(1/1!)
         .byte    0xFF
;
;
COSTBL:   ;
         .byte    0x37,0xD0,0x0D,0x01  ;       +(1/8!)
         .byte    0xBA,0xB6,0x0B,0x61  ;       -(1/6!)
         .byte    0x3D,0x2A,0xAA,0xAB  ;       +(1/4!)
         .byte    0xBF,0x00,0x00,0x00  ;       -(1/2!)
         .byte    0x3F,0x80,0x00,0x00  ;       +(1/1!)
         .byte    0xFF
;
;
__pi::
PI:       .byte    0x40,0x49,0x0F,0xDB  ;       3.1415927
N360:     .byte    0x43,0xB4,0x00,0x00   ;      360.0
;
;
;
;*****************************************************************************
;                                                                            *
;                        FLOATING POINT TANGENT                              *
;                                                                            *
;*****************************************************************************
;
;
_tan::
	pshb
	psha
	tsx
	ldab	#4
	abx
	jsr		__to_fpacc1
	bsr		FLTTAN
	tsx
	pulx
	jsr		__from_fpacc1
	rts
FLTTAN:   ;
         jsr    __push_fpacc2     ; SAVE FPACC2 ON THE STACK.
         jsr    __fp_xfer_acc1_to_acc2      ; PUT A COPY OF THE ANGLE IN FPACC2.
         jsr    FLTCOS       ; GET COSINE OF THE ANGLE.
         jsr    EXG1AND2     ; PUT RESULT IN FPACC2 & PUT ANGLE IN FPACC1.
         jsr    FLTSIN       ; GET SIN OF THE ANGLE.
         jsr    __fpdiv       ; GET TANGENT OF ANGLE BY DOING SIN/COS.
         bcc    FLTTAN1      ; IF CARRY CLEAR, ANSWER OK.
         jsr    __pull_fpacc2     ; RESTORE FPACC2
         ldaa   #TAN90ERR    ; GET ERROR CODE IN B.
         sec                 ; FLAG ERROR
         rts                 ; RETURN
FLTTAN1:  jsr    __pull_fpacc2    ;  RESTORE FPACC2.
         clrb                ; NO ERRORS
         rts                 ; RETURN.
;
;
__maxnum::
         .byte    0x7F,0xFF,0xFF,0xFF       ;  LARGEST POSITIVE NUMBER WE CAN HAVE.
;
;
;
;*****************************************************************************
;                                                                            *
;                              TRIG UTILITIES                                *
;                                                                            *
;        The routines "DEG2RAD" and "RAD2DEG" are used to convert angles     *
;        from degrees-to-radians and radians-to-degrees respectively. The    *
;        routine "GETPI" will place the value of PI into FPACC1. This        *
;        routine should be used if the value of PI is needed in calculations *
;        since it is accurate to the full 24-bits of the mantissa.           *
;                                                                            *
;*****************************************************************************
;

_degree_to_radian::
	pshb
	psha
	tsx
	ldab	#4
	abx
	jsr		__to_fpacc1
	bsr		DEG2RAD
	tsx
	pulx
	jsr		__from_fpacc1
	rts
DEG2RAD:  ;
         jsr    __push_fpacc2    ; SAVE FPACC2.
         ldx    #NPID180    ; POINT TO CONVERSION CONSTANT PI/180.
DEG2RAD1: jsr    __to_fpacc2   ;  PUT IT INTO FPACC2.
         jsr    __fpmul      ; CONVERT DEGREES TO RADIANS.
         jsr    __pull_fpacc2    ; RESTORE FPACC2.
         rts                ; RETURN. (NOTE! DON'T REPLACE THE "JSR/RTS" WITH
;                           ; A "JMP" IT WILL NOT WORK.)
;
;
_radian_to_degree::
	pshb
	psha
	tsx
	ldab	#4
	abx
	jsr		__to_fpacc1
	bsr		RAD2DEG
	tsx
	pulx
	jsr		__from_fpacc1
	rts
RAD2DEG:  ;
         jsr    __push_fpacc2    ; SAVE FPACC2.
         ldx    #N180DPI    ; POINT TO CONVERSION CONSTANT 180/PI.
         bra    DEG2RAD1    ; GO DO CONVERSION & RETURN.
;
;
GETPI:    ;
         ldx    #PI         ; POINT TO CONSTANT "PI".
         jmp    __to_fpacc1    ; PUT IT IN FPACC1 AND RETURN.
;
;
NPID180:  ;
         .byte    0x3C,0x8E,0xFA,0x31
;
N180DPI:  ;
         .byte    0x42,0x65,0x2E,0xE1
;
;
;
;*****************************************************************************
;                                                                            *
;        POLYNOM evaluates a polynomial with constant coefficients.  On      *
;        entry, FPACC1 contains the independent variable.  The X register    *
;        contains a pointer to a table of floating point coefficients,       *
;        stored with the highest order coefficient first.  The polynomial    *
;        is of arbitrary order; evaluation ends when 0xFF is encountered      *
;        after the last (lowest order) coefficient in the coefficient table. *
;                                                                            *
;*****************************************************************************
;
;
POLYNOM::
         xgdx               ; HOLD COEFFICIENT TABLE POINTER IN D
         ldx    __fpacc1_exp    ; SAVE F.P. ARGUMENT ON STACK (NOT MEMORY FORMAT!)
         pshx
         ldx    __fpacc1_mantissa+1
         pshx
         xgdx               ; GET COEFFICIENT TABLE POINTER BACK
         ldaa   __fpacc1_mantissa_sign    ; SAVE F.P. ARGUMENT SIGN ON STACK
         psha
         pshx               ; SAVE COEFFICIENT TABLE POINTER
         ldd    #0          ; CLEAR RESULT ACCUMULATOR
         std    __fpacc1_exp
         std    __fpacc1_mantissa+1
         std    __fpacc1_mantissa+2
         bra    POLY1
POLY2:    tsx               ;  GET POINTER TO COEFFICIENT TABLE POINTER
         ldaa   2,x         ; PUT INDEPENDENT VARIABLE INTO ACCUMULATOR 2
         staa   __fpacc2_mantissa_sign
         ldd    3,x
         std    __fpacc2_mantissa+1
         ldd    5,x
         std    __fpacc2_exp
         jsr    __fpmul      ; DO MULTIPLICATION
POLY1:    tsx               ;  GET POINTER TO COEFFICIENT TABLE POINTER
         ldx    0,x         ; GET POINTER TO COEFFICIENT TABLE
         jsr    __to_fpacc2    ; PUT COEFFICIENT INTO ACCUMULATOR
         jsr    __fpadd      ; ADD TO RESULT
         pulx               ; GET COEFFICIENT TABLE POINTER OFF STACK
         ldab   #4          ; INCREMENT COEFFICIENT POINTER
         abx
         pshx               ; SAVE NEW COEFFICIENT POINTER
         ldaa   0,x         ; CHECK FOR END OF TABLE
         coma               ; IF IT WAS 0xFF, WE ARE AT END
         bne    POLY2       ; NOT AT END - KEEP GOING
         tsx                ; DISCARD COEFFICIENT POINTER AND ARGUMENT
         ldab   #7          ; NUMBER OF BYTES TO PULL OFF STACK
         abx                ; ADD 7 TO STACK POINTER
         txs                ; PUT STACK POINTER BACK
         rts                ; DONE
